
/**
 * Copyright © pre 2008 by Linux Kernel Hackers
 * Copyright © 2008 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef BSWAP_H
#define BSWAP_H

#include "defs.h"
#include "haveinline.h"

/* Byte swap macros (based on linux/byteorder/swap.h) */
#define swap16(x) ((u16)( \
	(((u16)(x) & (u16)0x00ffU) << 8) | \
	(((u16)(x) & (u16)0xff00U) >> 8) ))

/* jfs needs swap24 */
#define swap24(x) ((u32)( \
	(((u32)(x) & (u32)0x000000ffUL) << 16) | \
	(((u32)(x) & (u32)0x0000ff00UL)      ) | \
	(((u32)(x) & (u32)0x00ff0000UL) >> 16) ))

#define swap32(x) ((u32)( \
	(((u32)(x) & (u32)0x000000ffUL) << 24) | \
	(((u32)(x) & (u32)0x0000ff00UL) <<  8) | \
	(((u32)(x) & (u32)0x00ff0000UL) >>  8) | \
	(((u32)(x) & (u32)0xff000000UL) >> 24) ))

#define swap64(x) ((u64)( \
	(u64)(((u64)(x) & (u64)0x00000000000000ffULL) << 56) | \
	(u64)(((u64)(x) & (u64)0x000000000000ff00ULL) << 40) | \
	(u64)(((u64)(x) & (u64)0x0000000000ff0000ULL) << 24) | \
	(u64)(((u64)(x) & (u64)0x00000000ff000000ULL) <<  8) | \
	(u64)(((u64)(x) & (u64)0x000000ff00000000ULL) >>  8) | \
	(u64)(((u64)(x) & (u64)0x0000ff0000000000ULL) >> 24) | \
	(u64)(((u64)(x) & (u64)0x00ff000000000000ULL) >> 40) | \
	(u64)(((u64)(x) & (u64)0xff00000000000000ULL) >> 56) ))

#if __BYTE_ORDER == __BIG_ENDIAN

/* __BIG_ENDIAN */
#define cpu_to_le16(x) swap16(x)
#define cpu_to_le24(x) swap24(x)
#define cpu_to_le32(x) swap32(x)
#define cpu_to_le64(x) swap64(x)
#define cpu_to_be16(x) (x)
#define cpu_to_be24(x) (x)
#define cpu_to_be32(x) (x)
#define cpu_to_be64(x) (x)
#define le16_to_cpu(x) swap16(x)
#define le24_to_cpu(x) swap24(x)
#define le32_to_cpu(x) swap32(x)
#define le64_to_cpu(x) swap64(x)
#define be16_to_cpu(x) (x)
#define be24_to_cpu(x) (x)
#define be32_to_cpu(x) (x)
#define be64_to_cpu(x) (x)

#else

/* __LITTLE_ENDIAN */
#define cpu_to_le16(x) (x)
#define cpu_to_le24(x) (x)
#define cpu_to_le32(x) (x)
#define cpu_to_le64(x) (x)
#define cpu_to_be16(x) swap16(x)
#define cpu_to_be24(x) swap24(x)
#define cpu_to_be32(x) swap32(x)
#define cpu_to_be64(x) swap64(x)
#define le16_to_cpu(x) (x)
#define le24_to_cpu(x) (x)
#define le32_to_cpu(x) (x)
#define le64_to_cpu(x) (x)
#define be16_to_cpu(x) swap16(x)
#define be24_to_cpu(x) swap24(x)
#define be32_to_cpu(x) swap32(x)
#define be64_to_cpu(x) swap64(x)

#endif				/* __BYTE_ORDER */

/**
 * some filesystems will use little or big endian
 */

#define BYTESEX_LE 0
#define BYTESEX_BE 1

static inline u64 fs64_to_cpu(int fs_endian, u64 n)
{
	return (fs_endian == BYTESEX_LE) ? le64_to_cpu(n) : be64_to_cpu(n);
}

static inline u64 cpu_to_fs64(int fs_endian, u64 n)
{
	return (fs_endian == BYTESEX_LE) ? cpu_to_le64(n) : cpu_to_be64(n);
}

static inline u32 fs32_to_cpu(int fs_endian, u32 n)
{
	return (fs_endian == BYTESEX_LE) ? le32_to_cpu(n) : be32_to_cpu(n);
}

static inline u32 cpu_to_fs32(int fs_endian, u32 n)
{
	return (fs_endian == BYTESEX_LE) ? cpu_to_le32(n) : cpu_to_be32(n);
}

static inline u16 fs16_to_cpu(int fs_endian, u16 n)
{
	return (fs_endian == BYTESEX_LE) ? le16_to_cpu(n) : be16_to_cpu(n);
}

static inline u16 cpu_to_fs16(int fs_endian, u16 n)
{
	return (fs_endian == BYTESEX_LE) ? cpu_to_le16(n) : cpu_to_be16(n);
}

#endif				/* BSWAP_H */
