
/**
 * Copyright © 2011 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* frisbee-ng */
#include "cdb.h"
#include "die.h"

/**
 * INQUIRY COMMAND (6)
 *
 * 0 = op code (0x12)
 * 1 = reserved
 * 2 = reserved
 * 3 = reserved
 * 4 = allocation len
 * 5 = reserved
 */

#define INQUIRY_SIZE  36	/* 8 + vendor(8) + model(16) + revision(4) */

void cdb_inquiry(int fd, char vendor[9], char model[17], char rev[5])
{
	cdb_t cdb;

	cdb_init(&cdb, fd, 6, INQUIRY_SIZE);

	/* send inquiry command */
	cdb.cmd.s[0] = 0x12;
	cdb.cmd.s[4] = INQUIRY_SIZE;
	cdb.flags = D_READ;

	/**
	 * device   vendor  model             rev
	 * /dev/sda ATA     TOSHIBA MK5055GS FG00
	 * /dev/sdb ATA     ST2000VM002-9UY1 SC01
	 * /dev/sdc Generic STORAGE DEVICE   9602
	 */
	if (cdb_sendcmd(&cdb) == -1) {

		/* we set some generic value ... */
		char v[] = { 'V', 'i', 'r', 't', 'u', 'a', 'l', 0 };
		char m[] = { 'S', 'T', 'O', 'R', 'A', 'G', 'E', ' ',
			'D', 'E', 'V', 'I', 'C', 'E', ' ', 0
		};
		char r[] = { '-', '-', '-', 0 };

		byte_copy(vendor, 8, v);
		byte_copy(model, 16, m);
		byte_copy(rev, 4, r);
		cdb_free(&cdb);

		return;
	}

	/* copy info to our struct */
	byte_copy(vendor, 8, cdb.buf.s + 8);
	vendor[8] = 0;

	byte_copy(model, 16, cdb.buf.s + 8 + 8);
	model[16] = 0;

	byte_copy(rev, 4, cdb.buf.s + 8 + 8 + 16);
	rev[4] = 0;

	cdb_free(&cdb);

	return;
}
