
/**
 * Copyright © 2008 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 8
 */

/* frisbee-ng */
#include "compr.h"

/**
 * compression types, which can be used @ frisbee
 */

char compr_usage_rep[] =
    "- alternative implementation of rzip by Bulat Ziganshin"
    "\nrep:b{N}:l{N}:d{N}:s{N}:h{N}:a{N}" "\n"
    "\nb{N} - use N MiB for sliding window (default: 128)"
    "\nl{N} - minimal match len (default: 512)"
    "\nd{N} - barrier for smaller matches (8MB)"
    "\ns{N} - minimal match len after barrier (0)"
    "\nh{N} - hash bits (default, calculated from size)"
    "\na{N} - coefficient of search amplification (default: 1)"
    "\n" "\ndefault: rep:bs128:mml128:ba8";

struct compression_type rep_compr = {
	.id = CFLAGS_REP,
	.name = "rep",
	.usage = compr_usage_rep
};

char compr_usage_delta[] = "- improve compression of tables in binary data";

struct compression_type delta_compr = {
	.id = CFLAGS_DELTA,
	.name = "delta",
	.usage = compr_usage_delta
};

char compr_usage_zstd[] =
    "- Zstandard" "\nzstd:l{N}" "\n" "\n  acceleration  desc"
    "\n           20:  slow" "\n           ..:  .." "\n           10:  default"
    "\n           ..:  .." "\n            1:  fast";

struct compression_type zstd_compr = {
	.id = CFLAGS_ZSTD,
	.name = "zstd",
	.usage = compr_usage_zstd
};

char compr_usage_none[] = "- no compression at all, just copy";

struct compression_type none_compr = {
	.id = CFLAGS_DATA,
	.name = "none",
	.usage = compr_usage_none,
};

char compr_usage_lzo[] =
    "- Lempel–Ziv–Oberhumer" "\nlzo:method-level" "\n"
    "\nvalid Mode-Level combinations:" "\nlzo:1-1 lzo:1-99"
    "\nlzo:1a-1 lzo:1a-99"
    "\nlzo:1b-1 lzo:1b-2 .. lzo:1b-9 lzo:1b-99 lzo:1b-999"
    "\nlzo:1c-1 lzo:1c-9 lzo:1c-99 lzo:1c-999" "\nlzo:1f-1 lzo:1f-999"
    "\nlzo:1x-1 lzo:1x-999" "\nlzo:1y-1 lzo:1y-999" "\nlzo:1z-999"
    "\nlzo:2a-999";

struct compression_type lzo_compr = {
	.id = CFLAGS_LZO,
	.name = "lzo",
	.usage = compr_usage_lzo
};

char compr_usage_lzf[] =
    "- an extremely fast compression algorithm" "\n" "\nno options";

struct compression_type lzf_compr = {
	.id = CFLAGS_LZF,
	.name = "lzf",
	.usage = compr_usage_lzf
};

char compr_usage_lz4[] =
    "- www.lz4.org, very fast decompression" "\nlz4:l{N}" "\n"
    "\n  acceleration  desc" "\n          200:  fastest"
    "\n           20:  fast (default)" "\n           ..:  .."
    "\n            1:  slowest";

struct compression_type lz4_compr = {
	.id = CFLAGS_LZ4,
	.name = "lz4",
	.usage = compr_usage_lz4
};

char compr_usage_lz4hc[] =
    "- the high-compression variant of LZ4" "\nlz4hc:l{N}" "\n"
    "\n  level  desc" "\n     0:  fastest (default)" "\n    ..:  .."
    "\n    16:  slowest";

struct compression_type lz4hc_compr = {
	.id = CFLAGS_LZ4HC,
	.name = "lz4hc",
	.usage = compr_usage_lz4hc
};

char compr_usage_lzjb[] = "- Lempel Ziv Jeff Bonwick" "\n" "\nno options";

struct compression_type lzjb_compr = {
	.id = CFLAGS_LZJB,
	.name = "lzjb",
	.usage = compr_usage_lzjb
};

char compr_usage_lzma[] =
    "- Lempel Ziv Markov chain Algorithm"
    "\nlzma:l{N}:d{N}:lc{N}:lp{N}:pb{N}:a{N}:fb{N}:m{N}:hb{N}\n" "\n"
    "\nl{N}  - level: 1..9 (def: 3)" "\nd{N}  - dictSize: 12..27 (2^N Bytes)"
    "\nlc{N} - literal context bits: 0..8 (def: 3)"
    "\nlp{N} - literal pos bits: 0..4 (def: 0)"
    "\npb{N} - number of pos bits: 0..4 (def: 0)"
    "\na{N}  - algorithm: 0=fast, 1=normal"
    "\nfb{N} - Word size: 5..273 (def: 32)" "\nm{N}  - btMode: 0, 1 (def: 0)"
    "\nhb{N} - hashbytes: 2..4 (def: 4)" "\n" "\n  level dictSize algo  fb"
    "\n     0:    16 KB    0  32" "\n     1:    64 KB    0  32"
    "\n     2:   256 KB    0  32" "\n     3:     1 MB    0  32"
    "\n     4:     4 MB    0  32" "\n     5:    16 MB    1  32"
    "\n     6:    32 MB    1  32" "\n     7:    64 MB    1  64";

struct compression_type lzma_compr = {
	.id = CFLAGS_LZMA,
	.name = "lzma",
	.usage = compr_usage_lzma
};

char compr_usage_gzip[] = "- DEFLATE compression algorithm";

struct compression_type gzip_compr = {
	.id = CFLAGS_GZIP,
	.name = "gzip",
	.usage = compr_usage_gzip
};

char compr_usage_bzip2[] =
    "- Burrows–Wheeler Algorithm 2" "\nbzip2:l{N}:wf{N}\n" "\n"
    "\nl{N}  - use level N, 0 <= N <= 9"
    "\nwf{N} - use workfactor N, 1 <= N <= 250" "\n" "\ndefault: bzip2:l3:wf30";

struct compression_type bzip2_compr = {
	.id = CFLAGS_BZIP2,
	.name = "bzip2",
	.usage = compr_usage_bzip2
};

char compr_usage_fastlz[] = "- Fast Lempel-Ziv";

struct compression_type fastlz_compr = {
	.id = CFLAGS_FASTLZ,
	.name = "fastlz",
	.usage = compr_usage_fastlz
};

char compr_usage_quicklz[] = "-  Quick Lempel-Ziv";

struct compression_type quicklz_compr = {
	.id = CFLAGS_QUICKLZ,
	.name = "quicklz",
	.usage = compr_usage_quicklz
};
