
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Derived from public domain C code in NACL, see http://nacl.cr.yp.to/
 * and https://131002.net/siphash/
 */

#ifndef crypto_auth_H
#define crypto_auth_H

#define crypto_auth_hmacsha512256_BYTES    32
#define crypto_auth_hmacsha512256_KEYBYTES 32
#define crypto_auth crypto_auth_hmacsha512256
#define crypto_auth_verify crypto_auth_hmacsha512256_verify
#define crypto_auth_BYTES    32
#define crypto_auth_KEYBYTES 32

#define crypto_auth_siphash24_BYTES    8
#define crypto_auth_siphash24_KEYBYTES 16

/* The crypto_auth function authenticates a message m[0], m[1], ..., m[mlen-1]
 * using a secret key k[0], k[1], ..., k[crypto_auth_KEYBYTES-1]. The
 * crypto_auth function puts the authenticator into a[0], a[1], ...,
 * a[crypto_auth_BYTES-1].
 *
 * It then returns 0.
 */
int crypto_auth_hmacsha512256(unsigned char *out, const unsigned char *in,
			      unsigned long long inlen, const unsigned char *k);

/* The crypto_auth_verify function returns 0 if a[0], ...,
 * a[crypto_auth_BYTES-1] is a correct authenticator of a message m[0], m[1],
 * ..., m[mlen-1] under a secret key k[0], k[1], ...,
 * k[crypto_auth_KEYBYTES-1].
 *
 * Otherwise crypto_auth_verify returns -1.
 */
int crypto_auth_hmacsha512256_verify(const unsigned char *h,
				     const unsigned char *in,
				     unsigned long long inlen,
				     const unsigned char *k);

int crypto_auth_siphash24(unsigned char *out /* 8 byte */ ,
			  const unsigned char *in, unsigned long long inlen,
			  const unsigned char *k /* 16 byte */ );

int crypto_auth_siphash24_verify(const unsigned char *h /* 8 byte */ ,
				 const unsigned char *in,
				 unsigned long long inlen,
				 const unsigned char *k /* 16 byte */ );

#endif
