
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Derived from public domain C code in NACL, see http://nacl.cr.yp.to/
 */

#include "crypto_box.h"
#include "crypto_core.h"
#include "crypto_scalarmult.h"
#include "crypto_secretbox.h"

#include "src/fmtX.h"

static const unsigned char sigma[16] = "expand 32-byte k";
static const unsigned char n[16] = { 0 };

int crypto_box_curve25519xsalsa20poly1305(unsigned char *c,
					  const unsigned char *m,
					  unsigned long long mlen,
					  const unsigned char *n,
					  const unsigned char *pk,
					  const unsigned char *sk)
{
	unsigned char k[32];

	crypto_box_curve25519xsalsa20poly1305_beforenm(k, pk, sk);

	return crypto_box_curve25519xsalsa20poly1305_afternm(c, m, mlen, n, k);
}

int crypto_box_curve25519xsalsa20poly1305_open(unsigned char *m,
					       const unsigned char *c,
					       unsigned long long clen,
					       const unsigned char *n,
					       const unsigned char *pk,
					       const unsigned char *sk)
{
	unsigned char k[32];

	crypto_box_curve25519xsalsa20poly1305_beforenm(k, pk, sk);

	return crypto_box_curve25519xsalsa20poly1305_open_afternm(m, c, clen, n,
								  k);
}

int crypto_box_curve25519xsalsa20poly1305_keypair(unsigned char *pk,
						  unsigned char *sk)
{
	fmt_urandom(sk, 32);

	return crypto_scalarmult_curve25519_base(pk, sk);
}

int crypto_box_curve25519xsalsa20poly1305_beforenm(unsigned char *k,
						   const unsigned char *pk,
						   const unsigned char *sk)
{
	unsigned char s[32];

	crypto_scalarmult_curve25519(s, sk, pk);

	return crypto_core_hsalsa20(k, n, s, sigma);
}

int crypto_box_curve25519xsalsa20poly1305_afternm(unsigned char *c,
						  const unsigned char *m,
						  unsigned long long mlen,
						  const unsigned char *n,
						  const unsigned char *k)
{
	return crypto_secretbox_xsalsa20poly1305(c, m, mlen, n, k);
}

int crypto_box_curve25519xsalsa20poly1305_open_afternm(unsigned char *m,
						       const unsigned char *c,
						       unsigned long long clen,
						       const unsigned char *n,
						       const unsigned char *k)
{
	return crypto_secretbox_xsalsa20poly1305_open(m, c, clen, n, k);
}
