
/**
 * Copyright © 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>

/* frisbee */
#include "src/defs.h"
#include "src/die.h"

#include "crypto_nonce.h"

typedef unsigned long long uint64;

static void uint64_pack(unsigned char *x, uint64 u)
{
	*x++ = u;
	u >>= 8;
	*x++ = u;
	u >>= 8;
	*x++ = u;
	u >>= 8;
	*x++ = u;
	u >>= 8;
	*x++ = u;
	u >>= 8;
	*x++ = u;
	u >>= 8;
	*x++ = u;
	u >>= 8;
	*x++ = u;
}

/* mutex for pid and fd_log */
static pthread_mutex_t nonce8_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t nonce24_mutex = PTHREAD_MUTEX_INITIALIZER;

static void die_nonce(void)
{
	fatal("NONCEMAX reached!");
}

/**
 * 2^64 - 6
 * -> should be okay
 */
#define NONCEMAX 18446744073709551610U

static u64 c = 0;

void crypto_nonce8_reset(void)
{
	pthread_mutex_lock(&nonce8_mutex);

	c = 0;

	pthread_mutex_unlock(&nonce8_mutex);

	return;
}

void crypto_nonce8(unsigned char *nonce)
{
	pthread_mutex_lock(&nonce8_mutex);

	if (unlikely(c == NONCEMAX))
		die_nonce();

	uint64_pack(nonce, c++);
	pthread_mutex_unlock(&nonce8_mutex);

	return;
}

void crypto_nonce8_u64(u64 * u)
{
	pthread_mutex_lock(&nonce8_mutex);

	if (unlikely(c == NONCEMAX))
		die_nonce();

	*u = c++;
	pthread_mutex_unlock(&nonce8_mutex);

	return;
}

static u64 c1 = 0, c2 = 0, c3 = 0;

void crypto_nonce24_reset(void)
{
	pthread_mutex_lock(&nonce24_mutex);

	c1 = c2 = c3 = 0;

	pthread_mutex_unlock(&nonce24_mutex);

	return;
}

void crypto_nonce24(unsigned char *nonce)
{
	pthread_mutex_lock(&nonce24_mutex);

	if (unlikely(c1 == NONCEMAX))
		die_nonce();

	if (c1 < c2)
		c1++;
	else if (c2 < c3)
		c2++;
	else
		c3++;

	uint64_pack(nonce, c1);
	uint64_pack(nonce + 8, c2);
	uint64_pack(nonce + 16, c3);

	pthread_mutex_unlock(&nonce24_mutex);

	return;
}
