
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Derived from public domain C code in NACL, see http://nacl.cr.yp.to/
 */

#ifndef crypto_onetimeauth_H
#define crypto_onetimeauth_H

#define crypto_onetimeauth crypto_onetimeauth_poly1305
#define crypto_onetimeauth_verify crypto_onetimeauth_poly1305_verify

/* The crypto_onetimeauth function authenticates a message m[0], m[1], ...,
 * m[mlen-1] using a secret key k[0], k[1], ...,
 * k[crypto_onetimeauth_KEYBYTES-1]; puts the authenticator into a[0], a[1],
 * ..., a[crypto_onetimeauth_BYTES-1]; and returns 0.
 */
int crypto_onetimeauth_poly1305(unsigned char *out, const unsigned char *in,
				unsigned long long inlen,
				const unsigned char *k);

/* This function returns 0 if a[0], a[1], ..., a[crypto_onetimeauth_BYTES-1]
 * is a correct authenticator of a message m[0], m[1], ..., m[mlen-1] under a
 * secret key k[0], k[1], ..., k[crypto_onetimeauth_KEYBYTES-1]. Otherwise
 * crypto_onetimeauth_verify returns -1.
 */
int crypto_onetimeauth_poly1305_verify(const unsigned char *h,
				       const unsigned char *in,
				       unsigned long long inlen,
				       const unsigned char *k);

#endif
