
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Derived from public domain C code in NACL, see http://nacl.cr.yp.to/
 */

#include "crypto_onetimeauth.h"
#include "crypto_auth.h"
#include "crypto_stream.h"
#include "crypto_secretbox.h"

/**
 * crypto_secretbox_xsalsa20poly1305(c,m,ml,n,k)
 *
 * @c: ciphertext (destination)
 * @m: message to encrypt, first 32 bytes are ZERO!
 * @ml: message length (m + 32)
 * @n: nonce (24 byte)
 * @k: shared secret key (32 byte)
 */
int crypto_secretbox_xsalsa20poly1305(unsigned char *c, const unsigned char *m,
				      unsigned long long mlen,
				      const unsigned char *n,
				      const unsigned char *k)
{
	unsigned int i;

	if (mlen < 32)
		return -1;
	crypto_stream_xsalsa20_xor(c, m, mlen, n, k);
	crypto_onetimeauth_poly1305(c + 16, c + 32, mlen - 32, c);
	for (i = 0; i < 16; ++i)
		c[i] = 0;
	return 0;
}

/**
 * crypto_secretbox_xsalsa20poly1305_open(c,m,ml,n,k)
 *
 * @m: plaintext message (destination)
 * @c: encrypted ciphertext, first 16 bytes are ZERO!
 * @ml: message length (mlen + 32)
 * @n: nonce (24 byte)
 * @k: shared secret key (32 byte)
 */
int crypto_secretbox_xsalsa20poly1305_open(unsigned char *m,
					   const unsigned char *c,
					   unsigned long long clen,
					   const unsigned char *n,
					   const unsigned char *k)
{
	unsigned int i;
	unsigned char subkey[32];

	if (clen < 32)
		return -1;
	crypto_stream_xsalsa20(subkey, 32, n, k);
	if (crypto_onetimeauth_poly1305_verify
	    (c + 16, c + 32, clen - 32, subkey) != 0)
		return -1;
	crypto_stream_xsalsa20_xor(m, c, clen, n, k);
	for (i = 0; i < 32; ++i)
		m[i] = 0;
	return 0;
}
