
/**
 * Copyright © 2012 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Derived from public domain C code in NACL and supercop, see
 * http://nacl.cr.yp.to/ and http://bench.cr.yp.to/supercop.html
 */

#ifndef crypto_stream_H
#define crypto_stream_H

#define crypto_stream crypto_stream_xsalsa20
#define crypto_stream_xor crypto_stream_xsalsa20_xor

#define crypto_stream_KEYBYTES   32
#define crypto_stream_NONCEBYTES 24	/* xsalsa routines */

/**
 * The crypto_stream function produces a stream c[0], c[1], ..., c[clen-1] as
 * a function of a secret key k[0], k[1], ..., k[crypto_stream_KEYBYTES-1] and
 * a nonce n[0], n[1], ..., n[crypto_stream_NONCEBYTES-1]. The crypto_stream
 * function then returns 0.
 *
 * The crypto_stream_xor function encrypts a message m[0], m[1], ...,
 * m[mlen-1] using a secret key k[0], k[1], ..., k[crypto_stream_KEYBYTES-1]
 * and a nonce n[0], n[1], ..., n[crypto_stream_NONCEBYTES-1]. The
 * crypto_stream_xor function puts the ciphertext into c[0], c[1], ...,
 * c[mlen-1]. It then returns 0.
 *
 * The crypto_stream_xor function guarantees that the ciphertext is the
 * plaintext xor the output of crypto_stream. Consequently crypto_stream_xor can
 * also be used to decrypt.
 */

/**
 * crypto_stream_xsalsa20()
 * nonce=24 rounds=20 key=32
 */
int crypto_stream_xsalsa20(unsigned char *c, unsigned long long clen,
			   const unsigned char *n, const unsigned char *k);

/**
 * crypto_stream_xsalsa20_xor()
 * nonce=24 rounds=20 key=32
 */
int crypto_stream_xsalsa20_xor(unsigned char *c, const unsigned char *m,
			       unsigned long long mlen, const unsigned char *n,
			       const unsigned char *k);

/**
 * crypto_stream_xsalsa208()
 * nonce=24 rounds=8 key=32
 */
int crypto_stream_xsalsa208(unsigned char *c, unsigned long long clen,
			    const unsigned char *n, const unsigned char *k);

/**
 * crypto_stream_xsalsa208_xor()
 * nonce=24 rounds=8 key=32
 */
int crypto_stream_xsalsa208_xor(unsigned char *c, const unsigned char *m,
				unsigned long long mlen, const unsigned char *n,
				const unsigned char *k);

/**
 * crypto_stream_salsa20()
 * nonce=8 rounds=20 key=32
 */
int crypto_stream_salsa20(unsigned char *c, unsigned long long clen,
			  const unsigned char *n, const unsigned char *k);

/**
 * crypto_stream_salsa20_xor()
 * nonce=8 rounds=20 key=32
 */
int crypto_stream_salsa20_xor(unsigned char *c, const unsigned char *m,
			      unsigned long long mlen, const unsigned char *n,
			      const unsigned char *k);

/**
 * crypto_stream_salsa208()
 * nonce=8 rounds=8 key=32
 */
int crypto_stream_salsa208(unsigned char *c, unsigned long long clen,
			   const unsigned char *n, const unsigned char *k);

/**
 * crypto_stream_salsa208_xor()
 * nonce=8 rounds=8 key=32
 */
int crypto_stream_salsa208_xor(unsigned char *c, const unsigned char *m,
			       unsigned long long mlen, const unsigned char *n,
			       const unsigned char *k);

#endif
