
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Derived from public domain C code in NACL, see http://nacl.cr.yp.to/
 */

#ifndef crypto_verify_H
#define crypto_verify_H

int crypto_verify_8(const unsigned char *x, const unsigned char *y);

/* The crypto_verify_16 function returns 0 if x[0], x[1], ..., x[15] are the
 * same as y[0], y[1], ..., y[15]. Otherwise it returns -1.
 * 
 * This function is safe to use for secrets x[0], x[1], ..., x[15], y[0], y[1],
 * ..., y[15]. The time taken by crypto_verify_16 is independent of the contents
 * of x[0], x[1], ..., x[15], y[0], y[1], ..., y[15]. In contrast, the standard
 * C comparison function memcmp(x,y,16) takes time that depends on the longest
 * matching prefix of x and y, often allowing easy timing attacks.
 */
int crypto_verify_16(const unsigned char *x, const unsigned char *y);

/* same with 32 bytes */
int crypto_verify_32(const unsigned char *x, const unsigned char *y);

#endif
