
/**
 * Copyright © 2005 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* system */
#include <pthread.h>
#include <stdarg.h>
#include <string.h>

/* libowfat */
#include "stralloc.h"
#include "str.h"
#include "scan.h"

/* frisbee-ng */
#include "strallocX.h"
#include "debug.h"
#include "defs.h"
#include "die.h"
#include "log.h"

/* don't mix messages for the global logbuffer from diff. threads */
static pthread_mutex_t dbg_mutex = PTHREAD_MUTEX_INITIALIZER;

/**
 * level: 1|2|4|8|16|32|64|128|256 => 1..9
 * section: 1..MAX_SECTION
 */
#if FULLDEBUG
unsigned int debugging[MAX_SECTION] = {
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9
};
#else
unsigned int debugging[MAX_SECTION] = {
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3
};
#endif

/**
 * - there only some minimum of formatting currently
 * - if we need more, adding some new features should be simple
 */
void debug_r(unsigned int section, int unsigned level, const char *fmt, ...)
{
	stralloc buf;		/* the filled buffer */
	va_list ap;

	/* should it be looged ? */
	if (debugging[section] < level)
		return;

	pthread_mutex_lock(&dbg_mutex);
	stralloc_init(&buf);

	va_start(ap, fmt);
	stralloc_fmt(&buf, fmt, ap);
	va_end(ap);

	log_sa_flush_nl(&buf);
	stralloc_free(&buf);
	pthread_mutex_unlock(&dbg_mutex);

	return;
}
