
/**
 * Copyright © 2008 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FNG_DEFS_H
#define FNG_DEFS_H

/* system */
#include <sys/types.h>
#include <inttypes.h>

/* I like the short types from kernel */
typedef uint8_t u8;		/* Unsigned types of an exact size */
typedef uint16_t u16;
typedef uint32_t u32;
typedef uint64_t u64;

typedef int8_t s8;		/* Signed types of an exact size */
typedef int16_t s16;
typedef int32_t s32;
typedef int64_t s64;

#if (__GNUC__ > 2)
#define likely(x)	__builtin_expect(!!(x),1)
#define unlikely(x)	__builtin_expect(!!(x),0)
#else
#define likely(x)	x
#define unlikely(x)	x
#endif				/* __builtin_expect */

#ifndef min
#define min(a,b)	((a) <= (b) ? (a) : (b))
#endif

#ifndef max
#define max(a,b)	((a) >= (b) ? (a) : (b))
#endif

/* setting some bit of some byte can be so easy ;) */
#define is_set(x,v)	(((x)&(v)) == (v))
#define set(x,v)	((x) |=  (v))
#define unset(x,v)	((x) &= ~(v))
#define toogle(x,v)	((x) ^=  (v))

/* unused variables should have a cast to void */
#define unused(x)	((void)(x))

/**
 * shut up fprintf()! /TR
 * - this way, we find also code, which uses stdio crap
 * - for testing only...
 */
#if 0
#define fprintf(dst,fmt,args...) do { } while (0)
#endif

/* test for handling type of packed structures */
#include "haveattrpacked.h"
#include "havepragmapack.h"

/* we use a lot inlining */
#include "haveinline.h"

#if defined(HAVE_ATTR_PACKED)
 /* we can use __attribute__((packed)), no need for #pragma pack() */
#undef HAVE_PRAGMA_PACK
#define __FNG_ATTR_PACKED __attribute__((packed))
#elif defined(HAVE_PRAGMA_PACK)
 /* we have to use #pragma pack() stuff */
#define __FNG_ATTR_PACKED
#else
#error "I don't know how to handle packing alignment?"
#endif

/* work around some 32bit bug... */
#ifdef __dietlibc__
#warning "dietlibc's pthreads seem to be broken"
#define pwrite pwrite64
#define pread pread64
#endif

#endif
