
/**
 * Copyright © 2008 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FNG_DEV_H
#define FNG_DEV_H

/**
 * these funtions may need to be ported to other systems ;)
 */

/* libowfat */
#include "stralloc.h"
#include "defs.h"

/**
 * get all harddisk devices we can access and return them in one stralloc
 * - not to much intelligence here... try and fail is okay!
 * - no loop devices (use shell scripts!)
 * - no device mapper stuff (use shell scripts!)
 *
 * @return: "/dev/sda\0/dev/sdb\0.../dev/hdz\0"
 */
stralloc *dev_get_disks(void);

/**
 * get all network cards the system has
 * - not to much intelligence here... try and fail is okay!
 * - hopefully a bit portable via libowfat ^^
 *
 * @return: "[unknown]\0wlan0\0lo\0eth0\0"
 *
 * - this way, we can switch between the interfaces (scope_id) and are
 *   compatible with socket_getifidx() ;)
 */
stralloc *dev_get_nics(int *count);

/**
 * dev_getdisksize - return disksize in bytes
 */
extern u64 dev_getdisksize(int fd);

/**
 * dev_getsectorsize - return logical sector size of disk
 */
extern int dev_getsectorsize(int fd);

/**
 * dev_reread_ptable - re-read the partition table of disk
 */
extern void dev_reread_ptable(int fd);

/**
 * dev_create_loop - create loop device, and return the fd
 *
 * @ffd: fd of some file or device, to loop from
 * @name: name to use
 * @offset: offset, which should be mappped
 * @size: size to map
 *
 * @return: new fd, of some loop/map device
 */
extern int dev_loop_create(int ffd, char *name, u64 offset, u64 size);

/**
 * dev_loop_close - close loop device created with dev_loop_create()
 */
extern void dev_loop_close(int lfd);

#endif				/* FNG_DEV_H */
