
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef __DragonFly__

/* system */
#include <sys/ioctl.h>
#include <sys/diskslice.h>

/* libowfat */
#include "stralloc.h"

/* frisbee-ng */
#include "strallocX.h"
#include "term.h"
#include "defs.h"
#include "dev.h"
#include "die.h"
#include "mem.h"

static int testdisk_and_add(const char *path);

static int testdisk_and_add(const char *path)
{
	struct stat st;
	int r;

	r = lstat(path, &st);
	if (r == -1)
		return 0;	/* not found or no permission */

	/* no duplication of the same disk */
	if (!S_ISLNK(st.st_mode))
		return 0;

	return 1;
}

/**
 * get all harddisk devices we can access and return them in one stralloc
 *
 * @return: "???"
 */
stralloc *dev_get_disks(void)
{
	stralloc *rv, sa;
	int i, j;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, ""))
		die_nomem();

	rv = mem_talloc(stralloc, 1);	/* return value */
	stralloc_init(rv);
	if (!stralloc_copys(rv, ""))
		die_nomem();

	for (i = 0; i < 32; i++) {

		/* IDE disk devices */
		if (!stralloc_copys(&sa, "/dev/ad"))
			die_nomem();
		if (!stralloc_cat_uint(&sa, i))
			die_nomem();
		if (!stralloc_0(&sa))
			die_nomem();
		if (testdisk_and_add(sa.s)) {
			if (!stralloc_cat(rv, &sa))
				die_nomem();
		}

		/* SCSI disk devices */
		if (!stralloc_copys(&sa, "/dev/da"))
			die_nomem();
		if (!stralloc_cat_uint(&sa, i))
			die_nomem();
		if (!stralloc_0(&sa))
			die_nomem();
		if (testdisk_and_add(sa.s)) {
			if (!stralloc_cat(rv, &sa))
				die_nomem();
		}

		/* SATA disk devices */
		if (!stralloc_copys(&sa, "/dev/ada"))
			die_nomem();
		if (!stralloc_cat_uint(&sa, i))
			die_nomem();
		if (!stralloc_0(&sa))
			die_nomem();
		if (testdisk_and_add(sa.s)) {
			if (!stralloc_cat(rv, &sa))
				die_nomem();
		}

	}
	stralloc_free(&sa);

	return rv;
}

/**
 * dev_getdisksize - return disksize in bytes
 */
u64 dev_getdisksize(int fd)
{
	struct partinfo dp;
	int r;

	r = ioctl(fd, DIOCGPART, &dp);
	if (r == 0)
		return dp.media_size;

	/* may be a file? */
	if (r == -1) {
		off_t o = lseek(fd, 0, SEEK_END);

		if (o != (off_t) - 1) {
			bytes = o;
			return bytes;
		}
	}

	die_msg(111, "ioctl(DIOCGPART) and lseek(fd) failed!");
	return 0;		/* not reached */
}

/**
 * dev_getsectorsize - return logical sector size of disk
 */
int dev_getsectorsize(int fd)
{
	struct partinfo dp;
	int r;

	r = ioctl(fd, DIOCGPART, &dp);
	if (r == 0)
		return dp.media_blksize;

	return 512;
}

/**
 * dev_reread_ptable - re-read the partition table of disk
 */
void dev_reread_ptable(int fd)
{
	return;			/* is there sth. we can do? */
}

#endif
