
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* libowfat */
#include "str.h"
#include "socket.h"
#include "stralloc.h"

/* frisbee-ng */
#include "defs.h"
#include "dev.h"
#include "die.h"
#include "mem.h"

/**
 * get all network cards the system has
 * - not to much intelligence here... try and fail is okay!
 * - hoipefully a bit portable via libowfat ^^
 *
 * @return: "[unknown]\0wlan0\0lo\0eth0\0"
 *
 * - this way, we can switch between the interfaces (scope_id) and are
 *   compatible with socket_getifidx() ;)
 */
stralloc *dev_get_nics(int *count)
{
	stralloc *rv;
	int i;

	rv = mem_talloc(stralloc, 1);	/* return value */
	stralloc_init(rv);
	*count = 0;

	/* 0 ist immer [unknown], kommt auch in den index */
	for (i = 0; i < 32; i++) {
		const char *x = socket_getifname(i);

		if (str_equal(x, "[unknown]"))
			break;
		if (!stralloc_cats(rv, x))
			die_nomem();
		if (!stralloc_0(rv))
			die_nomem();
		*count += 1;
	}

	return rv;
}
