
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef __OpenBSD__

/*

openbsd/sys/dev/vnd.c
openbsd/sys/dev/vndioctl.h

*/

/* system */
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/disklabel.h>
#include <sys/dkio.h>
#include <sys/stat.h>

/* libowfat */
#include "stralloc.h"

/* frisbee-ng */
#include "strallocX.h"
#include "term.h"
#include "defs.h"
#include "dev.h"
#include "die.h"
#include "mem.h"

static int testdisk_and_add(const char *path);

static int testdisk_and_add(const char *path)
{
	struct stat st;
	int r;

	r = lstat(path, &st);
	if (r == -1)
		return 0;	/* not found or no permission */
	if (!S_ISBLK(st.st_mode))
		return 0;	/* not an block device */

	return 1;
}

/**
 * get all harddisk devices we can access and return them in one stralloc
 *
 * @return: "/dev/sd0\0/dev/sd1\0 ..."
 */
stralloc *dev_get_disks(void)
{
	stralloc *rv, sa;
	int i, j;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, ""))
		die_nomem();

	rv = mem_talloc(stralloc, 1);	/* return value */
	stralloc_init(rv);
	if (!stralloc_copys(rv, ""))
		die_nomem();

	for (i = 0; i < 32; i++) {
		char x = 'a' + i;

		/* IDE, SATA, MFM, ESDI disk devices */
		if (!stralloc_copys(&sa, "/dev/wd"))
			die_nomem();
		if (!stralloc_catb(&sa, &x, 1))
			die_nomem();
		if (!stralloc_0(&sa))
			die_nomem();
		if (testdisk_and_add(sa.s)) {
			if (!stralloc_cat(rv, &sa))
				die_nomem();
		}

		/* SCSI, USB, SATA, AHCI, RAID disk devices */
		if (!stralloc_copys(&sa, "/dev/sd"))
			die_nomem();
		if (!stralloc_catb(&sa, &x, 1))
			die_nomem();
		if (!stralloc_0(&sa))
			die_nomem();
		if (testdisk_and_add(sa.s)) {
			if (!stralloc_cat(rv, &sa))
				die_nomem();
		}
	}

	stralloc_free(&sa);

	return rv;
}

/**
 * dev_getdisksize - return disksize in bytes
 */
u64 dev_getdisksize(int fd)
{
	struct stat st;
	int r;

	r = fstat(fd, &st);
	if (r != 0)
		die_msg(111, "fstat(fd) failed!");

	if (S_ISCHR(st.st_mode) || S_ISBLK(st.st_mode)) {
		struct disklabel dl;

		r = ioctl(fd, DIOCGDINFO, &dl);
		if (r != 0)
			die_msg(111, "ioctl(DIOCGDINFO) failed!");

		return dl.d_secsize *
		    dl.d_partitions[DISKPART(st.st_rdev)].p_size;
	}

	return st.st_size;
}

/**
 * dev_getsectorsize - return logical sector size of disk
 */
int dev_getsectorsize(int fd)
{
	/* XXX */
	return 512;
}

/**
 * re-read partition table
 */
void dev_reread_ptable(int fd)
{
	return;
}

int dev_loop_create(int ffd, char *name, u64 offset, u64 size)
{
	return 0;
}

void dev_loop_close(int lfd)
{
	return;
}

#endif
