
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef __sun__

/* system */
#include <sys/dkio.h>

/* libowfat */
#include "stralloc.h"

/* frisbee-ng */
#include "strallocX.h"
#include "term.h"
#include "defs.h"
#include "dev.h"
#include "die.h"
#include "mem.h"

static int testdisk_and_add(const char *path);

static int testdisk_and_add(const char *path)
{
	struct stat st;
	int r;

	r = lstat(path, &st);
	if (r == -1)
		return 0;	/* not found or no permission */
	if (!S_ISBLK(st.st_mode))
		return 0;	/* not an block device */

	return 1;
}

/**
 * get all harddisk devices we can access and return them in one stralloc
 *
 * @return: "/dev/ad0\0/dev/da0\0 ..."
 */
stralloc *dev_get_disks(void)
{
	stralloc *rv, sa;
	int i, j;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, ""))
		die_nomem();

	rv = mem_talloc(stralloc, 1);	/* return value */
	stralloc_init(rv);
	if (!stralloc_copys(rv, ""))
		die_nomem();

	for (i = 0; i < 26; i++) {	/* 'a'..'z' */
		char x = 'a' + i;

		/* SCSI, SAS and SATA disk devices */
		if (!stralloc_copys(&sa, "/dev/sd"))
			die_nomem();
		if (!stralloc_catb(&sa, &x, 1))
			die_nomem();
		if (!stralloc_0(&sa))
			die_nomem();
		if (testdisk_and_add(sa.s)) {
			if (!stralloc_cat(rv, &sa))
				die_nomem();
		}
	}

	stralloc_free(&sa);

	return rv;
}

/**
 * dev_getdisksize - return disksize in bytes
 */
u64 dev_getdisksize(int fd)
{
	struct dk_minfo minfo;
	int r;

	r = ioctl(fd, DKIOCGMEDIAINFO, &minfo);
	if (r == 0)
		return minfo.dki_lbsize * minfo.dki_capacity;

	/* may be a file? */
	if (r == -1) {
		off_t o = lseek(fd, 0, SEEK_END);

		if (o != (off_t) - 1) {
			bytes = o;
			return bytes;
		}
	}

	die_msg(111, "ioctl(DKIOCGMEDIAINFO) and lseek(fd) failed!");
	return 0;		/* not reached */
}

/**
 * dev_getsectorsize - return logical sector size of disk
 */
int dev_getsectorsize(int fd)
{
	struct dk_minfo minfo;
	int r;

	r = ioctl(fd, DKIOCGMEDIAINFO, &minfo);
	if (r == 0)
		return minfo.dki_lbsize;

	return 512;
}

/**
 * re-read partition table
 */
void dev_reread_ptable(int fd)
{
	return;
}

#endif
