
/**
 * Copyright © 2008 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* system */
#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>

/* libowfat */
#include "errmsg_int.h"
#include "fmt.h"

/* frisbee-ng */
#include "defs.h"
#include "term.h"
#include "tty.h"
#include "die.h"
#include "debug.h"
#include "main.h"

void __die_msg(int r, const char *message, ...)
{
	va_list a;

	va_start(a, message);

	/* informational -> stdout */
	if (r == 0) {
		errmsg_write(1, 0, message, a);
	} else if (errno == 0) {
		/* return value != 0 -> stderr */
		errmsg_write(2, 0, message, a);
	} else {
		/* stderr + error string of errno */
		errmsg_write(2, strerror(errno), message, a);
	}

	fng_exit(r);
}

void die_io(void)
{
	die_msg(111, "io");
}

void die_guru(void)
{
	die_msg(111, "☯ Guru Meditation ☯");
}				/* mail me, when this happens! */

void die_term(const char *m)
{
	die_msg(111, m);
}

void die_rzip(const char *m)
{
	die_msg(111, m);
}

void die_pread(const char *fs)
{
	die_msg(111, "pread failed @ fs=", fs);
}

void die_nomem(void)
{
	die_msg(111, "nomem");
}

void die_perm(void)
{
	die_msg(111, "permission");
}

void die_crypto(void)
{
	die_msg(111, "crypt");
}

void _die_pthread(int r)
{
	errno = r;
	die_msg(111, "pthread");
}

void die_lseek(size_t pos, const char *at)
{
	char fmt[FMT_ULONG];

	fmt[fmt_long(fmt, pos)] = 0;
	die_msg(111, "lseek to pos=", fmt, " failed @ ", at);
}

void die_compr(const char *msg, int ret)
{
	char fmt[FMT_LONG];

	fmt[fmt_long(fmt, ret)] = 0;
	die_msg(3, msg, fmt);
}
