
/**
 * Copyright © 2011 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* system */
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>

/* frisbee-ng */
#include "die.h"
#include "debug.h"
#include "drop_privs.h"

#define NAMEBUF 500
static void xgetpwnam(const char *name, gid_t * gid, uid_t * uid)
{
	struct passwd *pw;

	pw = getpwnam(name);
	if (!pw) {
		die_msg(111, "Could not lookup user %s", name);
	}

	*gid = pw->pw_gid;
	*uid = pw->pw_uid;

	return;
}

int drop_privs(const char *svr_user, const char *svr_dir)
{
	uid_t uid;
	gid_t gid;

	/* we are not root, so just run in current directory with current uid/gid */
	if (geteuid() != (uid_t) 0)
		return 0;

	/* look up id's, we should run as */
	xgetpwnam(svr_user, &gid, &uid);

	/* 1) chroot */
	if (svr_dir)
		if (chroot(svr_dir) != 0)
			die_msg(111, "Could not chroot to %s", svr_user);

	if (chdir("/") != 0)
		fatal("chdir(/) failed, after chrooting: ");

	/* 2) drop rights */
	setgid(gid);
	setuid(uid);

	return 0;
}
