
/**
 * Copyright © 2001 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FMTX_H
#define FMTX_H

/**
 * some extensions to the standard fmt routines
 */

#include "fmt.h"

/**
 * 2^128 + dots # '\0' => 52
 * echo -n  "340.282.366.920.938.463.463.374.607.431.768.211.456"|wc -c
 */

#define FMT_ULONG_DOT 52

extern size_t fmt_x64pw(char *dst, const unsigned long long i,
			const unsigned int pad, const char with);

extern size_t fmt_u64pw(char *dst, const unsigned long long i,
			const unsigned int pad, const char with);

extern size_t fmt_s64pw(char *dst, const long long i, const unsigned int pad,
			const char with);

extern size_t fmt_dotted(char *dst, const unsigned long long i, const char dot);

extern size_t fmt_uuid(char *dst, const char i[16]);

/**
 * append n random bytes to char *dst
 * - based on xsalsa20/8 by DJB
 */
extern size_t fmt_random(char *dst, int n);
extern size_t fmt_urandom(unsigned char *dst, int n);

extern size_t fmt_zero(char *dst, int n);
extern size_t fmt_uzero(unsigned char *dst, int n);

extern size_t fmt_padleft(char *dst, size_t srclen, size_t padlen,
			  size_t maxlen, char with);

extern size_t fmt_padright(char *dst, const char *src, size_t srclen,
			   size_t padlen, size_t maxlen, char with);

/**
 * tu_date - return fixed size date string
 * format: yyyy-mm-dd
 */
extern size_t fmt_date(char *dest, time_t t);

/**
 * tu_date - return fixed size time string
 * format: mm:hh:ss
 */
extern size_t fmt_time(char *dest, time_t t);

/**
 * tu_date - return fixed size datetime string
 * format: yyyy-mm-dd mm:hh:ss
 */
extern size_t fmt_datetime(char *dest, time_t t);

#endif				/* FMTX_H */
