
/**
 * Copyright © 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <time.h>

#include "fmtX.h"

/**
 * fmt_date - return fixed size date string
 * format: yyyy-mm-dd
 */
size_t fmt_date(char *dest, time_t t)
{
	struct tm *tm = localtime(&t);
	size_t r = 0;

	if (!dest)
		return 4 + 1 + 2 + 1 + 2;

	r += fmt_u64pw(dest + r, tm->tm_year + 1900, 4, '0');
	r += fmt_str(dest + r, "-");
	r += fmt_u64pw(dest + r, tm->tm_mon + 1, 2, '0');
	r += fmt_str(dest + r, "-");
	r += fmt_u64pw(dest + r, tm->tm_mday, 2, '0');

	return r;
}

/**
 * fmt_time - return fixed size time string
 * format: mm:hh:ss
 */
size_t fmt_time(char *dest, time_t t)
{
	struct tm *tm = localtime(&t);
	size_t r = 0;

	if (!dest)
		return 2 + 1 + 2 + 1 + 2;

	r += fmt_u64pw(dest + r, tm->tm_hour, 2, '0');
	r += fmt_str(dest + r, ":");
	r += fmt_u64pw(dest + r, tm->tm_min, 2, '0');
	r += fmt_str(dest + r, ":");
	r += fmt_u64pw(dest + r, tm->tm_sec, 2, '0');

	return r;
}

/**
 * fmt_datetime - return fixed size datetime string
 * format: yyyy-mm-dd mm:hh:ss
 */
size_t fmt_datetime(char *dest, time_t t)
{
	struct tm *tm = localtime(&t);
	size_t r = 0;

	if (!dest)
		return 4 + 1 + 2 + 1 + 2 + 1 + 2 + 1 + 2 + 1 + 2;

	r += fmt_u64pw(dest + r, tm->tm_year + 1900, 4, '0');
	r += fmt_str(dest + r, "-");
	r += fmt_u64pw(dest + r, tm->tm_mon + 1, 2, '0');
	r += fmt_str(dest + r, "-");
	r += fmt_u64pw(dest + r, tm->tm_mday, 2, '0');
	r += fmt_str(dest + r, " ");
	r += fmt_u64pw(dest + r, tm->tm_hour, 2, '0');
	r += fmt_str(dest + r, ":");
	r += fmt_u64pw(dest + r, tm->tm_min, 2, '0');
	r += fmt_str(dest + r, ":");
	r += fmt_u64pw(dest + r, tm->tm_sec, 2, '0');

	return r;
}
