
/**
 * Copyright © 2012 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FNG_ADHOC_H
#define FNG_ADHOC_H

/* system */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>

/* libowfat */
#include "socket.h"
#include "buffer.h"
#include "scan.h"
#include "open.h"
#include "io.h"
#include "str.h"
#include "ip4.h"
#include "ip6.h"

/* frisbee-ng */
#include "strallocX.h"
#include "term.h"
#include "crypt.h"
#include "compr.h"
#include "main.h"
#include "tty.h"
#include "dev.h"
#include "die.h"
#include "getkey.h"
#include "cdb.h"
#include "log.h"
#include "wrapper.h"
#include "list.h"
#include "bitmap.h"
#include "sc.h"
#include "fs.h"
#include "pt_reader.h"
#include "sysstat.h"
#include "openX.h"
#include "progress.h"
#include "mem.h"

#define FNG_ADHOC_CLIENT        1
#define FNG_ADHOC_SERVER_DISK   2
#define FNG_ADHOC_SERVER_PART   3

struct adhoc_config {
	/* default values, user may change them... */
	int mtu;
	int queuelen;
	int blks_chunk;

	/**
	 * compression type and options to use
	 */
	stralloc compr;
	u32 compr_count;

	/* /etc/services: 7983-7997   Unassigned */
	int fng_port;
	char fng_bcip[4];
	char *mip;
	char fng_mcip[16];

	stralloc group;
	stralloc devices;
	u32 devcount;
};

struct chunk {
	u64 idx_chunk;		/* number of chunk @ partition */
	u64 idx_block;		/* number of first block */
	u32 st_blocks;		/* stored blocks in this chunk */

#define CHUNK_UNUSED     0x0000
#define CHUNK_USED       0x0001
	u32 flags;		/* used for the state of *data */

	void *data;
};

/* each partition gets it's own todo structure */
struct todo {
	u32 index;		/* index number in todo strcture */
	u32 number;		/* slice number, 1,2,5,6 */
	u64 start;		/* offset @ disk */
	u64 size;		/* size on disk */
	struct fsinfo fsi;	/* filesystem info */

	struct chunk *c;
	u64 clen;
};

/* generic handling structure for server */
struct sstate {
	struct adhoc_config *cfg;	/* for easy access of the configuration */
	int64 fd;		/* opened device in read only mode */
	char *device;		/* path to device or file */
	char *group;		/* name of multicast group */
	u64 min_devsize;	/* needed disksize in bytes */

	struct slice *sl;	/* slice information of device */
	u32 slen;		/* number of slices in *sl */
	char *type;		/* name of partition type (gpt, bsd, mbr...) */
	int ptype;		/* id of partition type, see pt_reader.h  */

	struct todo *todo;	/* each slice gets it's own thing here */
	u32 tlen;		/* number of todo's */

	int64 fd_bc;		/* socket for broadcast */
	int64 fd_mc;		/* socket for multicast */

	/* special static packet data */
	stralloc p_hello;
	stralloc p_config;
	stralloc p_ptable;

	/* pointer to queuelen allocated chunks */
	u64 *chunks;
	int fifo;
};

/* generic handling structure for client */
struct cstate {
	struct adhoc_config *cfg;	/* for easy access of the configuration */
	int64 fd;		/* opened in rw mode */
	char *path;		/* path to device or file */
	u64 devsize;		/* disksize in bytes */

#define STATE_HELLO      0
#define STATE_NETCFG     1
#define STATE_BOOTCFG    2
#define STATE_PARTCFG    3
#define STATE_DATA       4
	int state;

	/* xxx: add last N chunk buffers / or packets */
	progress_t p;		/* progress for cloning on this disk */

	/* special static packet data */
	stralloc p_hello;
	stralloc p_config;
	stralloc p_ptable;
};

#endif
