
/**
 * Copyright © 2010 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 1
 */

/* system */
#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

/* libowfat */
#include "array.h"
#include "stralloc.h"
#include "strallocX.h"
#include "textcode.h"
#include "scan.h"
#include "str.h"
#include "io.h"
#include "ip4.h"
#include "ip6.h"
#include "socket.h"
#include "byte.h"
#include "open.h"
#include "scan.h"
#include "fmt.h"
#include "buffer.h"
#include "io.h"
#include "errmsg.h"

/* frisbee */
#include "log.h"
#include "die.h"
#include "list.h"
#include "sc.h"
#include "errmsg.h"
#include "sig.h"
#include "main.h"
#include "mem.h"
#include "drop_privs.h"
#include "term.h"
#include "crypt.h"
#include "tty.h"
#include "getblock.h"

static void usage(void)
{
	term_print("\nUsage:\n");
	term_print(" fng-client [options]\n");
	term_print("\nGeneral Options:\n");
	term_print(" %16s%s", "-h or -?", "show help\n");
	term_print(" %16s%s", "-v", "version\n");
	term_print("\nApplication Options:\n");
	term_print(" %16s%s", "-p", "port\n");
	term_print(" %16s%s", "-m", "mcast ip\n");
	term_print(" %16s%s", "-d", "server directory\n");
	term_print(" %16s%s", "-u", "username\n");
	fng_exit(0);
}

/* create detached threads by default */
static pthread_attr_t attr_detached;

/**
 * mainloop of the server
 */
static void mainloop(void);

static void mainloop(void)
{
	int fd = open_read("lala");

	if (fd == -1)
		fng_exit(1);

	term_printflush("ende gelände....\n");
	fng_exit(0);

	return;
}

int fng_client(int argc, char **argv)
{
	char *fng_server_serveruser = "nobody";
	char *fng_server_serverdir = ".";
	int opt;
	unsigned x;
	char mip[16];

	/* initialize some attribute for creating detached threads */
	pthread_attr_init(&attr_detached);
	pthread_attr_setdetachstate(&attr_detached, PTHREAD_CREATE_DETACHED);

	byte_zero(mip, sizeof(mip));

	while ((opt = getopt(argc, argv, "h?vm:p:d:u:")) != -1) {
		switch (opt) {
		case 'h':	/* help */
		case '?':
			usage();
		case 'v':	/* version */
			version();
#if 0
		case 'm':	/* multicast ip for clients */
			r = scan_ip6(optarg, mip);
			if (r != (int)str_len(optarg))
				die_msg(0, lang_msg(MSG_WRONG_IP));
			break;
		case 'p':	/* udp port */
			r = scan_ushort(optarg, &port);
			if (r != (int)str_len(optarg))
				die_msg(0, lang_msg(MSG_WRONG_PORT));
			break;
#endif
		case 'd':	/* chroot directory, default: "." */
			fng_server_serverdir = optarg;
			break;
		case 'u':	/* username, default: nobody */
			fng_server_serveruser = optarg;
			break;
		default:
			usage();
		}
	}

	return fng_exit(0);

	// XXX
	for (x = 0; x < 32; x++)
		term_print("r=%u 1<<%u=%u\n", x, x, 1 << x);
	term_flush();
	return 0;

	/* drop root privs + chroot() */
	drop_privs(fng_server_serveruser, fng_server_serverdir);

	tty_init();

	/* enter the main loop, which waits for io */
	mainloop();

	return fng_exit(0);
}
