
/**
 * Copyright © 2010 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 1
 */

/* system */
#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

/* libowfat */
#include "array.h"
#include "stralloc.h"
#include "strallocX.h"
#include "textcode.h"
#include "scan.h"
#include "str.h"
#include "io.h"
#include "ip4.h"
#include "ip6.h"
#include "socket.h"
#include "byte.h"
#include "open.h"
#include "scan.h"
#include "fmt.h"
#include "buffer.h"
#include "io.h"
#include "errmsg.h"

/* frisbee */
#include "tty.h"
#include "log.h"
#include "die.h"
#include "list.h"
#include "sc.h"
#include "errmsg.h"
#include "sig.h"
#include "main.h"
#include "mem.h"
#include "drop_privs.h"
#include "term.h"
#include "crypt.h"

/**
 * !server has to be stateless!
 */

static void usage(void)
{
	term_print("\nUsage:\n");
	term_print(" fng-server [options]\n");
	term_print("\nGeneral Options:\n");
	term_print(" %16s%s", "-h or -?", "show help\n");
	term_print(" %16s%s", "-v", "version\n");
	term_print("\nApplication Options:\n");
	term_print(" %16s%s", "-p", "port (default 7983)\n");
	term_print(" %16s%s", "-m", "mcast ip (default 224.102.110.103)\n");
	term_print(" %16s%s", "-d", "server directory (default '.')\n");
	term_print(" %16s%s", "-u", "username (default: nobody)\n");
	term_print(" %16s%s", "-n", "server name (Frisbee-NG Server)\n");
	fng_exit(0);
}

/* create detached threads by default */
static pthread_attr_t attr_detached;

/**
 * mainloop of the server
 */
static void mainloop(char *s);

static void mainloop(char *s)
{
	term_print("server: %s", s);

	return;
}

int fng_server(int argc, char **argv)
{
	char *fng_server_serveruser = "nobody";
	char *fng_server_serverdir = ".";
	char *fng_server_name;
	int opt, r;

	char mip[16];
	u16 port = 7983;

	/* initialize some attribute for creating detached threads */
	pthread_attr_init(&attr_detached);
	pthread_attr_setdetachstate(&attr_detached, PTHREAD_CREATE_DETACHED);

	byte_zero(mip, sizeof(mip));

	/* the default multicast ip from params.h */
	r = scan_ip6("1.2.3.4", mip);
	if (r != (int)str_len("2.3.4.5")) {
		fatal("rad ip");
	}

	/* default: "FrisbeeNG" */
	fng_server_name = "Frisbee-NG Server";

	while ((opt = getopt(argc, argv, "h?vn:m:p:d:u:")) != -1) {
		switch (opt) {
		case 'h':	/* help */
		case '?':
			usage();
		case 'v':	/* version */
			version();
		case 'n':	/* server name */
			fng_server_name = optarg;
			break;
		case 'm':	/* multicast ip for image group */
			r = scan_ip6(optarg, mip);
			if (r != (int)str_len(optarg))
				fatal("ip");
			break;
		case 'p':	/* port */
			r = scan_ushort(optarg, &port);
			if (r != (int)str_len(optarg))
				fatal("port");
			break;
		case 'd':	/* chroot directory, default: "." */
			fng_server_serverdir = optarg;
			break;
		case 'u':	/* username, default: nobody */
			fng_server_serveruser = optarg;
			break;
		default:
			usage();
		}
	}

	/* drop root privs + chroot() */
	drop_privs(fng_server_serveruser, fng_server_serverdir);

	tty_init();

	/* enter the main loop, which waits for io */
	mainloop(fng_server_name);

	return fng_exit(0);
}
