
/**
 * Copyright © 2010 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * initial code was taken from util-linux-2.10s
 * Copyright © Andries Brouwer <aeb@cwi.nl> and <util-linux@math.uio.no>
 */

/* system */
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

/* frisbee-ng */
#include "wrapper.h"
#include "term.h"
#include "mem.h"
#include "getblock.h"

/**
 * small cache for disk reading - no threading here!
 */
struct block {
	off_t secnr;
	u8 *block;
	struct block *next;
};

static struct block *blockhead;

#define BSSIZE 512
u8 *getblock(int fd, off_t secnr)
{
	struct block *bp;

	for (bp = blockhead; bp; bp = bp->next) {
		if (bp->secnr == secnr)
			return bp->block;
	}

	/* create new entry */
	bp = mem_tzalloc(struct block, 1);
	bp->block = mem_tzalloc(u8, BSSIZE);
	if (pread_loop(fd, bp->block, BSSIZE, secnr * BSSIZE) != BSSIZE) {
		/* not as much data as wanted... */
		mem_free(bp->block);
		mem_free(bp);
		return 0;
	}

	bp->secnr = secnr;
	bp->next = blockhead;
	blockhead = bp;

	/* return data */
	return bp->block;
}

/* free all cached disk reads */
void getblock_free(void)
{
	struct block *bp = blockhead;

	while (bp) {
		struct block *tmp = bp->next;

		mem_free(bp->block);
		mem_free(bp);
		bp = tmp;
	}
	blockhead = 0;

	return;
}
