
/**
 * Copyright © 2009 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* system */
#include <string.h>

/* libowfat */
#include "buffer.h"
#include "io.h"
#include "str.h"
#include "byte.h"

/* fribee-ng */
#include "getkey.h"
#include "defs.h"
#include "term.h"
#include "wrapper.h"
#include "mem.h"
#include "die.h"

#define GETKEY_MAXLEN 10

int getkey_simple(char *keyset)
{
	char ch;
	unsigned int i;

	io_fd2(0);
	io_nonblock(0);

	for (;;) {
		io_wantread(0);
		io_wait();
		io_dontwantread(0);

		if (buffer_getc(buffer_0, &ch) != 1)
			for (i = 0; i < str_len(keyset); i++)
				if (keyset[i] == ch)
					return i;
	}

	die_msg(111, "getkey_simple() sucks!");

	/* never reached */
	return 0;
}

int getkey(keyset_t * keyset)
{
	char *ustr = mem_alloc(GETKEY_MAXLEN + 1);
	unsigned int i;
	int rlen;

	io_fd2(0);
	io_nonblock(0);

	for (;;) {
		byte_zero(ustr, GETKEY_MAXLEN + 1);
		io_wantread(0);
		io_wait();
		io_dontwantread(0);
		if (buffer_get(buffer_0, ustr, GETKEY_MAXLEN) == 0)
			continue;
		rlen = str_len(ustr);
		for (i = 0; keyset[i].index != -1; i++) {
			int rlen_k = str_len(keyset[i].keystr);

			if (rlen == rlen_k)
				if (byte_equal(keyset[i].keystr, rlen, ustr)) {
					mem_free(ustr);
					return keyset[i].index;
				}
		}
	}
	mem_free(ustr);

	return -1;
}

int getkey_nonblocking(keyset_t * keyset)
{
	char *ustr = mem_alloc(GETKEY_MAXLEN + 1);
	unsigned int i;

	io_fd2(0);
	io_nonblock(0);

	for (;;) {
		int rlen, r;

		byte_zero(ustr, GETKEY_MAXLEN + 1);
		r = buffer_get(buffer_0, ustr, GETKEY_MAXLEN);
		rlen = str_len(ustr);
		for (i = 0; keyset[i].index != -1; i++) {
			int rlen_k = str_len(keyset[i].keystr);

			if (rlen == rlen_k)
				if (byte_equal(keyset[i].keystr, rlen, ustr)) {
					mem_free(ustr);
					return keyset[i].index;
				}
		}

		/* unknown key */
		if (r == -1)
			break;
	}
	mem_free(ustr);

	return -1;
}
