
/**
 * Copyright © 2009 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FNG_GETKEY_H
#define FNG_GETKEY_H

/**
 * getkey_old - read a key from non-blocking standard input
 *
 * @param keyset a set of possible keys, which are accepted
 *
 * @return - -1 on error or the index of the found key in keyset
 */
extern int getkey_simple(char *keyset);

typedef struct {
	/* we have an extra index, so we can define multiple strings for e.g. "yes" */
	int index;		/* last one: -1 */
	char *keystr;		/* last one: NULL */
} keyset_t;

/**
 * getkey - read a key from non-blocking standard input
 * - will wait for correct input, so it is some art of blocking
 *
 * @param keyset a set of possible keys, which are accepted
 *
 * @return
 *  on success: the index of the found key in keyset
 *  on error: -1
 */
extern int getkey(keyset_t * keyset);
extern int getkey_nonblocking(keyset_t * keyset);

#if 0
	/* sample: */
keyset_t k[] = {
	{1, "ö"},
	{2, "ä"},
	{3, "ß"},
	{4, "J"},
	{4, "j"},
	{5, "n"},
	{5, "N"},
	{5, "€"},
	{5, "¢"},
	{-1, NULL},
};

	//term_print("type one key of abc!\n");
	//opt = getkey_old("abc"); // opt = 0-2

	//term_print("type one key of öäßJjnN€¢!\n");
	//opt = getkey(k); // opt = index
	//term_print("key = %d \n", opt);

#endif

#endif
