
/**
 * lzma_lib.h  -- LZMA library wrapper
 *
 * Copyright © 1999-2008 Igor Pavlov
 * Copyright © 2008-2009 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.
 */

/* system */
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* frisbee-ng */
#include "lzma_enc.h"
#include "lzma_dec.h"
#include "lzma_lib.h"

static void *SzAlloc(void *p, size_t size)
{
	(void)p;
	return malloc(size);
}

static void SzFree(void *p, void *address)
{
	(void)p;
	free(address);
}
static ISzAlloc g_Alloc = { SzAlloc, SzFree };

int LzmaCompress(unsigned char *dst, size_t * dstlen, const unsigned char *src,
		 size_t srclen, int level, unsigned int dictSize, int lc,
		 int lp, int pb, int algo, int fb, int btMode, int numHashBytes)
{
	CLzmaEncProps props;
	size_t outPropsSize = LZMA_PROPS_SIZE;

	/* dst = pbuf[5] + cbuf[dstlen-5] */
	unsigned char *cbuf, *pbuf;
	const unsigned char *ubuf;
	size_t clen, ulen;
	int ret;

	(void)dictSize;

	LzmaEncProps_Init(&props);
	props.level = level;
	if (lc)
		props.lc = lc;
	if (lp)
		props.lp = lp;
	if (pb)
		props.pb = pb;
	if (algo)
		props.algo = algo;
	if (fb)
		props.fb = fb;
	if (btMode)
		props.btMode = btMode;
	if (numHashBytes)
		props.numHashBytes = numHashBytes;

	pbuf = dst;		/* first 5 bytes of dst */
	cbuf = dst + LZMA_PROPS_SIZE;	/* the rest */
	ubuf = src;

	clen = *dstlen - LZMA_PROPS_SIZE;
	ulen = srclen;

	/* encode the data */
	ret =
	    LzmaEncode(cbuf, &clen, ubuf, ulen, &props, pbuf, &outPropsSize, 1,
		       NULL, &g_Alloc, &g_Alloc);

	*dstlen = clen + LZMA_PROPS_SIZE;

	return ret;
}

int LzmaUncompress(unsigned char *dst, size_t * dstlen,
		   const unsigned char *src, size_t * srclen)
{
	ELzmaStatus status;
	const unsigned char *cbuf, *pbuf;
	unsigned char *ubuf;
	size_t clen, ulen;

	cbuf = src + LZMA_PROPS_SIZE;
	pbuf = src;
	ubuf = dst;

	clen = *srclen - LZMA_PROPS_SIZE;
	ulen = *dstlen;

	return LzmaDecode(ubuf, &ulen, cbuf, &clen, pbuf, LZMA_PROPS_SIZE,
			  LZMA_FINISH_END, &status, &g_Alloc);
}
