
/**
 * Copyright © 2010 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "wrapper.h"
#include "die.h"
#include "term.h"

s64 pread_loop(int fd, void *buffer, u64 count, u64 offset)
{
	s64 done;

	done = 0;
	while (count > 0) {
		s64 block = pread(fd, buffer + done, count,
				  offset + done);

		if (block < 0)
			return block;
		if (!block)
			return done;

		done += block;
		count -= block;
	}

	return done;
}

s64 pread_loop2(int fd, void *buffer, u64 count, u64 offset)
{
	s64 r = pread_loop(fd, buffer, count, offset);

	if (r != (s64) count)
		die_msg(111, "short read @ pread_loop()");

	return r;
}

s64 pread_sa(int fd, stralloc * sa, u64 count, u64 offset)
{
	s64 r;

	if (!stralloc_readyplus(sa, count))
		die_nomem();

	r = pread_loop2(fd, sa->s + sa->len, count, offset);
	sa->len += r;

	return r;
}
