
/**
 * Copyright © 2009 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Debug Section: 16
 */

/* frisbee-ng */
#include "progress.h"
#include "defs.h"
#include "die.h"
#include "term.h"

static void die_progress(u64 current, u64 total)
{
	//debug(16, 1, "p->current=%D > p->total=%D ?!\n", current, total);
	die_msg(111, "progress failure!");
}

void progress_init(progress_t * p, u64 total)
{
	pthread_mutex_init(&p->mutex, 0);

	/* current time = start time alias t0 */
	gettimeofday(&p->t0, 0);

	p->total = total;
	p->current = 0;
	p->ctt_h = p->ctt_m = p->ctt_s = 0;
	p->eta_h = p->eta_m = p->eta_s = 0;
	p->percent = 0;
	p->timediff = 0;
	p->datadiff = 0;

	return;
}

void progress_initadd(progress_t * p, u64 add)
{
	/* current time = start time alias t0 */
	gettimeofday(&p->t0, 0);

	p->total += add;

	return;
}

void progress_update(progress_t * p, u64 current)
{
	pthread_mutex_lock(&p->mutex);

	p->current = current;
	if (p->current > p->total)
		die_progress(p->current, p->total);

	pthread_mutex_unlock(&p->mutex);

	return;
}

void progress_add(progress_t * p, u64 add)
{
	pthread_mutex_lock(&p->mutex);

	p->current += add;

	if (p->current > p->total)
		die_progress(p->current, p->total);

	pthread_mutex_unlock(&p->mutex);

	return;
}

int progress(progress_t * p)
{
	int x;

	pthread_mutex_lock(&p->mutex);

	/* get current time */
	gettimeofday(&p->tc, 0);

	p->timediff = p->tc.tv_sec - p->t0.tv_sec;

	/* max one times a second! */
	if (p->timediff <= 1) {
		pthread_mutex_unlock(&p->mutex);
		/* we set some defaults... so there will not come FPE */
		p->timediff = 1;
		return 0;
	}

	p->datadiff = p->total - p->current;
	if (p->current > p->total) {
		die_progress(p->current, p->total);
	}

	if (p->current == 0) {
		pthread_mutex_unlock(&p->mutex);
		return 0;	/* not pssible to calculate sth. */
	}

	/**
	 * current        = xxx%
	 * p->st_clusters = 100%
	 */
	x = (p->current * 100) / p->total;
#if 0
	if (x == p->percent) {
		pthread_mutex_unlock(&p->mutex);
		return 0;	/* change to small, at least 1% */
	}
#endif
	p->percent = x;

	/* total time */
	p->ctt_s = p->timediff;
	p->ctt_h = p->ctt_s / 3600;
	p->ctt_s -= p->ctt_h * 3600;
	p->ctt_m = p->ctt_s / 60;
	p->ctt_s -= p->ctt_m * 60;

	/* estimated time of arrival */
	p->eta_s = (p->timediff * p->total) / p->current;
	p->eta_s -= p->timediff;
	p->eta_h = p->eta_s / 3600;
	p->eta_s -= p->eta_h * 3600;
	p->eta_m = p->eta_s / 60;
	p->eta_s -= p->eta_m * 60;

	pthread_mutex_unlock(&p->mutex);

	return 1;
}
