
/**
 * Copyright © 2009 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef PROGRESS_H
#define PROGRESS_H

/* system */
#include <sys/types.h>
#include <sys/time.h>
#include <pthread.h>

/* ntfs-3g */
#include "defs.h"		/* u16 .. u64 defined here */

struct progress {
	pthread_mutex_t mutex;	/* mutex, if used by multiple threads */
	struct timeval t0;	/* will be set by progress_init() */
	struct timeval tc;	/* will be set by progress_update() */
	u64 total;		/* sum of all */
	u64 current;		/* current of total, must be <= total */

	int ctt_h, ctt_m, ctt_s;	/* current total time */
	int eta_h, eta_m, eta_s;	/* estimated time */
	int percent;		/* 0..100 */
	int timediff;		/* current tc-t0 */
	int datadiff;		/* current total-current */
};

typedef struct progress progress_t;

/* initialize progress structure */
extern void progress_init(progress_t * p, u64 total);
extern void progress_initadd(progress_t * p, u64 add);

/* set current counter to sth. */
extern void progress_update(progress_t * p, u64 current);

/* add sth. to current counter */
extern void progress_add(progress_t * p, u64 add);

/* calculate times and stuff
 * @return 0 - unchanged
 * @return 1 - changed values, you can redraw...
 */
extern int progress(progress_t * p);

#endif
