
/**
 * Copyright © 2005 Bastian Blank
 * Copyright © 2011 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Initially taken from multipath-tools, which derived from util-linux :)
 */

/* system */
#include <inttypes.h>
#include <string.h>

/* libowfat */
#include "byte.h"

/* frisbee-ng */
#include "bswap.h"
#include "pt_reader.h"

int is_extended(int type)
{
	return (type == 5 || type == 0xf || type == 0x85);
}

static int read_extended_partition(int fd, struct mbrpart *ep, int en,
				   struct slice *sp, int ns)
{
	struct mbrpart p;
	unsigned long start, here, next;
	u8 *bp;
	int loopct = 0;
	int moretodo = 1;
	int i, n = 0;

	next = start = le32_to_cpu(ep->start_sect);
	while (moretodo) {
		here = next;
		moretodo = 0;
		if (++loopct > 100)
			return n;

		bp = (u8 *) getblock(fd, here);
		if (bp == NULL)
			return n;

		if (bp[510] != 0x55 || bp[511] != 0xaa)
			return n;

		for (i = 0; i < 2; i++) {
			byte_copy(&p, sizeof(p), bp + 0x1be + i * sizeof(p));
			if (is_extended(p.sys_type)) {
				if (p.nr_sects && !moretodo) {
					next =
					    start + le32_to_cpu(p.start_sect);
					moretodo = 1;
				}
				continue;
			}
			if (n < ns) {
				sp[n].start = here + le32_to_cpu(p.start_sect);
				sp[n].size = le32_to_cpu(p.nr_sects);
				sp[n].container = en + 1;
				n++;
			} else {
				return n;
			}
			loopct = 0;
		}
	}

	return n;
}

int read_dos_pt(int fd, struct slice *sp, int ns)
{
	struct mbrpart p;
	int i, n = 4;
	u8 *bp;

	bp = (u8 *) getblock(fd, 0);
	if (bp == NULL)
		return -1;

	if (bp[510] != 0x55 || bp[511] != 0xaa)
		return -1;

	for (i = 0; i < 4; i++) {
		byte_copy(&p, sizeof(p), bp + 0x1be + i * sizeof(p));

		if (p.sys_type == 0xee) {
			return 0;	/* gpt */
		}

		if (i < ns) {
			sp[i].start = le32_to_cpu(p.start_sect);
			sp[i].size = le32_to_cpu(p.nr_sects);
			sp[i].container = p.sys_type;
		} else
			break;

		if (is_extended(p.sys_type)) {
			n += read_extended_partition(fd, &p, i, sp + n, ns - n);
		}
	}

	return n;
}
