
/**
 * Copyright © 2012 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* libowfat */
#include "stralloc.h"
#include "io.h"
#include "uint16.h"
#include "uint32.h"
#include "uint64.h"

/* frisbee-ng */
#include "pt_reader.h"
#include "term.h"
#include "mem.h"
#include "die.h"
#include "wrapper.h"
#include "bswap.h"

/*
 * XXX: take care for grub and so on...
 *
 * 20480 * BLKSZ = 10 MiB
 */

#define BLKSZ 512
static void pt_image_gpt(int fd, u64 devsize, u32 * len,
			 struct ptable_data **pd)
{
	struct ptable_data *p;
	char *gpt;

	gpt = mem_alloc(BLKSZ * 34);
	p = mem_tzalloc(struct ptable_data, 2);

	*len = 2;
	*pd = p;

	/* gpt am anfang lesen */
	pread_loop2(fd, gpt, BLKSZ * 34, 0);
	p[0].start = 0;
	stralloc_init(&p[0].data);
	if (!stralloc_copyb(&p[0].data, gpt, BLKSZ * 34))
		die_nomem();

	/* gpt am ende lesen */
	pread_loop2(fd, gpt, BLKSZ * 34, devsize - BLKSZ * 34);
	stralloc_init(&p[1].data);
	if (!stralloc_copyb(&p[1].data, gpt, BLKSZ * 34))
		die_nomem();
	p[1].start = devsize - BLKSZ * 34;

	mem_free(p);
	mem_free(gpt);

	return;
}

static void pt_image_dos(int fd, u32 * len, struct ptable_data **pd)
{
	struct ptable_data *p;
	char *mbr = mem_alloc(BLKSZ);
	struct mbrpart *pt;
	void *x;
	int i;
	u32 lba, start;

	/* bringt uns nicht um, wird später wieder frei gegeben...  */
	p = mem_tzalloc(struct ptable_data, 65);

	*len = 1;		/* erstmal 1, wird aber passend hoch gezählt... */
	*pd = p;

	/* 1) copy mbr - this is easy ;) */
	pread_loop2(fd, mbr, BLKSZ, 0);
	p[0].start = 0;
	stralloc_init(&p[0].data);
	if (!stralloc_copyb(&p[0].data, mbr, BLKSZ))
		die_nomem();

	/* 2) search for ebr's and copy them... */
	lba = 0;
	for (i = 0; i < 4; i++) {
		x = mbr + 446 + 16 * i;
		pt = x;

		if (is_extended(pt->sys_type)) {
			lba = pt->start_sect;
			break;
		}
	}

	/* no extended partition found */
	if (lba == 0)
		return;

	for (start = 0, i = 0; i < 64; i++) {
		u16 sig;

		if (lba == 0)
			break;

		pread_loop2(fd, mbr, BLKSZ, (start + lba) * BLKSZ);
		p[*len].start = (start + lba) * BLKSZ;
		stralloc_init(&p[*len].data);
		if (!stralloc_copyb(&p[*len].data, mbr, BLKSZ))
			die_nomem();
		*len += 1;

		/* have we found our way out? */
		uint16_unpack(mbr + 510, &sig);
		if (sig != 0xaa55)
			break;

		/* reset to go away value */
		if (start == 0)
			start = lba;
		lba = 0;

		x = mbr + 446;
		pt = x;
		if (is_extended(pt->sys_type))
			lba = pt->start_sect;

		x = mbr + 446 + 16;
		pt = x;
		if (is_extended(pt->sys_type))
			lba = pt->start_sect;
	}

	return;
}

/**
 * pt_image() - image partiton table for later restoration
 *
 * @fd: source disk to read from
 * @ptype: type of partition table (gpt, dos, bsd, ...)
 * @len: number of data entries written to pd
 * @pd: the data for imaging
 */
void pt_image(int fd, int ptype, u64 devsize, u32 * len,
	      struct ptable_data **pd)
{
	if (ptype == PTYPE_GPT)
		pt_image_gpt(fd, devsize, len, pd);
	else if (ptype == PTYPE_DOS)
		pt_image_dos(fd, len, pd);
	else
		fatal("Only mbr/gpt are properly restored currently!");

	/* I don't use BSD or other systems...
	 * if you need it, send be a patch ;)
	 */

	return;
}
