
/**
 * Copyright © 2011 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Initially taken from multipath-tools, which derived from util-linux :)
 */

/* system */
#include <sys/types.h>

/* frisbee-ng */
#include "bswap.h"
#include "pt_reader.h"

#define MAC_PARTITION_MAGIC     0x504d

/* type field value for A/UX or other Unix partitions */
#define APPLE_AUX_TYPE  "Apple_UNIX_SVR2"

#ifdef HAVE_PRAGMA_PACK
#pragma pack(push,1)
#endif

struct mac_partition {
	uint16_t signature;	/* expected to be MAC_PARTITION_MAGIC */
	uint16_t res1;
	uint32_t map_count;	/* # blocks in partition map */
	uint32_t start_block;	/* absolute starting block # of partition */
	uint32_t block_count;	/* number of blocks in partition */
	/* there is more stuff after this that we don't need */
};

#define MAC_DRIVER_MAGIC        0x4552

/* Driver descriptor structure, in block 0 */
struct mac_driver_desc {
	uint16_t signature;	/* expected to be MAC_DRIVER_MAGIC */
	uint16_t block_size;
	uint32_t block_count;
};

#ifdef HAVE_PRAGMA_PACK
#pragma pack(pop)
#endif

int read_mac_pt(int fd, struct slice *sp, int ns)
{
	struct mac_driver_desc *md;
	struct mac_partition *part;
	unsigned secsize;
	u8 *data;
	int blk, blocks_in_map;
	int n = 0;

	md = (struct mac_driver_desc *)getblock(fd, 0);
	if (md == 0)
		return -1;

	if (be16_to_cpu(md->signature) != MAC_DRIVER_MAGIC)
		return -1;

	secsize = be16_to_cpu(md->block_size);
	data = getblock(fd, secsize / 512);
	if (!data)
		return -1;
	part = (struct mac_partition *)(data + secsize % 512);

	if (be16_to_cpu(part->signature) != MAC_PARTITION_MAGIC)
		return -1;

	blocks_in_map = be32_to_cpu(part->map_count);
	for (blk = 1; blk <= blocks_in_map && blk <= ns; ++blk, ++n) {
		int pos = blk * secsize;

		data = getblock(fd, pos / 512);
		if (!data)
			return -1;

		part = (struct mac_partition *)(data + pos % 512);
		if (be16_to_cpu(part->signature) != MAC_PARTITION_MAGIC)
			break;

		sp[n].start = be32_to_cpu(part->start_block) * (secsize / 512);
		sp[n].size = be32_to_cpu(part->block_count) * (secsize / 512);
	}

	return n;
}
