
/**
 * Copyright © 2012 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* frisbee-ng */
#include "pt_reader.h"
#include "term.h"
#include "mem.h"

#define MAXSLICES 260

struct pt {
	char *type;
	ptreader *fn;
} pts[KNOWN_PT_READER];

static void addpt(int n, char *t, ptreader f)
{
	pts[n].type = t;
	pts[n].fn = f;
}

static void initpts(void)
{
	static int done = 0;

	if (done)
		return;

	addpt(0, "gpt", read_gpt_pt);
	addpt(1, "dos", read_dos_pt);
	addpt(2, "bsd", read_bsd_pt);
	addpt(3, "solaris", read_solaris_pt);
	addpt(4, "mac", read_mac_pt);
	addpt(5, "sun", read_sun_pt);
	addpt(6, "unixware", read_unixware_pt);

	done = 1;

	return;
}

/**
 * pt_getlist() - read partiton list of disk and return it
 *
 * return type of partitiontable or 0 on error
 */
int pt_getlist(int fd, struct slice **sl, u32 * slen, char **type)
{
	struct slice *slices, *rv;
	struct pt *ptp;
	u32 i, n, r, ptype = 0;

	/* free old cached data */
	getblock_free();

	initpts();
	slices = mem_tzalloc(struct slice, MAXSLICES);

	for (i = 0; i < KNOWN_PT_READER; i++) {
		ptp = &pts[i];
		n = ptp->fn(fd, slices, MAXSLICES);
		if (n > 0) {
			*type = ptp->type;
			ptype = i + 1;
			break;
		}
	}

	/* free cached data, we may use other fd's later... */
	getblock_free();

	if (n <= 0)
		return 0;

	/* count used slices */
	*slen = 0;
	for (i = 0; i < n; i++) {
		if (slices[i].size == 0)
			continue;
		*slen += 1;
	}

	/* allocate slice array and fill it */
	rv = mem_tzalloc(struct slice, *slen);

	for (i = 0, r = 0; i < n; i++) {
		if (slices[i].size == 0)
			continue;
		rv[r].number = i + 1;
		rv[r].start = slices[i].start;
		rv[r].size = slices[i].size;
		rv[r].container = slices[i].container;
		r += 1;
	}
	mem_free(slices);

	*sl = rv;

	return ptype;
}
