
/**
 * Copyright © 2010 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "wrapper.h"
#include "die.h"

#ifdef __dietlibc__
#define pwrite pwrite64
#endif

s64 pwrite_loop(int fd, const void *buffer, u64 count, u64 offset)
{
	s64 done = 0;

	while (count > 0) {
		s64 block = pwrite(fd, buffer + done, count,
				   offset + done);

		if (block < 0)
			return block;
		if (!block)
			return done;

		done += block;
		count -= block;
	}

	return done;
}

s64 pwrite_loop2(int fd, const void *buffer, u64 count, u64 offset)
{
	s64 r = pwrite_loop(fd, buffer, count, offset);

	if (r != (s64) count)
		die_msg(111, "short write @ pwrite_loop()");

	return r;
}

s64 pwrite_sa(int fd, stralloc * sa, u64 offset)
{
	return pwrite_loop2(fd, sa->s, sa->len, offset);
}
