
/**
 * Copyright © 2012 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/* libowfat */
#include "open.h"
#include "byte.h"

/* frisbee */
#include "defs.h"
#include "die.h"
#include "havecpuid.h"
#include "randombytes.h"

#ifdef HAVE_CPUID

static inline int rdrand64(unsigned long long *v)
{
#if 0
	unsigned char err;
	asm volatile
	 (".byte 0x48; .byte 0x0f; .byte 0xc7; .byte 0xf0; setc %1":"=a" (*v),
	  "=qm"(err));

	return (int)err;
#endif
	unused(v);
	return 0;
}

void rdrand_bytes(unsigned char *x, unsigned long long xlen)
{
	void *dest = x;

	while (xlen > 0) {
		unsigned long long *v;
		int r;

		if (xlen < 8)
			v = dest;
		else
			v = dest + (xlen - 8);

		r = rdrand64(v);
		if (r == 0) {
			die_msg(111, "rdrand64() = 0");
		}

		xlen -= 8;
	}
}

/**
 * return 1, if current cpu has on-chip DRNG support
 */
int have_rdrand_bytes(void)
{
#if 0
	unsigned int eax = 1;
	unsigned int ebx = 0;
	unsigned int ecx = 0;
	unsigned int edx = 0;

	asm volatile (".byte 15;.byte 162"	/* cpuid */
		      :"=a" (eax), "=b"(ebx), "=c"(ecx), "=d"(edx):"0"(eax),
		      "2"(ecx):"memory");

	if (is_set(ecx, 1 << 30))
		return 1;
#endif

	return 0;
}
#endif

void randombytes(unsigned char *x, unsigned long long xlen)
{
	static int fd = -1;
	ssize_t i;

#ifdef HAVE_CPUID
	/* -1 = init / 0 = does not have / 1 = have it */
	static int have_rdrand = -1;

	if (have_rdrand == -1)
		have_rdrand = have_rdrand_bytes();

	if (have_rdrand == 1)
		return rdrand_bytes(x, xlen);
#endif

	if (fd == -1) {
		for (;;) {
			fd = open_read("/dev/urandom");
			if (fd != -1)
				break;
			sleep(1);
		}
	}

	while (xlen > 0) {
		if (xlen < 1048576)
			i = xlen;
		else
			i = 1048576;

		i = read(fd, x, i);
		if (i < 1) {
			sleep(1);
			continue;
		}

		x += i;
		xlen -= i;
	}

	/* we leave fd open, for the next time */
}
