
/**
 * Copyright © 2008 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 12
 */

/* system */
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

/* libowfat */
#include "buffer.h"
#include "open.h"
#include "fmt.h"
#include "str.h"
#include "errmsg.h"

/* fribee-ng */
#include "list.h"
#include "compr.h"

extern struct compression_type none_compr;
extern struct compression_type lzo_compr;
extern struct compression_type lzf_compr;
extern struct compression_type lz4_compr;
extern struct compression_type lz4hc_compr;
extern struct compression_type lzjb_compr;
extern struct compression_type lzma_compr;
extern struct compression_type gzip_compr;
extern struct compression_type bzip2_compr;
extern struct compression_type fastlz_compr;
extern struct compression_type quicklz_compr;
extern struct compression_type rep_compr;
extern struct compression_type delta_compr;
extern struct compression_type zstd_compr;

LIST_HEAD(compression_types);

static void register_compression(struct compression_type *ct)
{
	INIT_LIST_HEAD(&ct->list);
	list_add_tail(&ct->list, &compression_types);

	return;
}

void register_compression_modes(void)
{
	/* hier wird auch die reihenfolge für '-h' definiert ... */

	/* pre compression */
	register_compression(&rep_compr);
	register_compression(&delta_compr);

	/* no levels */
	register_compression(&none_compr);
	register_compression(&lzo_compr);
	register_compression(&lzf_compr);
	register_compression(&lzjb_compr);
	register_compression(&quicklz_compr);

	/* with levels */
	register_compression(&lz4hc_compr);
	register_compression(&lz4_compr);
	register_compression(&lzma_compr);
	register_compression(&gzip_compr);
	register_compression(&bzip2_compr);
	register_compression(&fastlz_compr);
	register_compression(&zstd_compr);

	return;
}
