
/**
 * Copyright © 2008 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 12
 */

/* system */
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

/* libowfat */
#include "buffer.h"
#include "open.h"
#include "fmt.h"
#include "str.h"
#include "errmsg.h"

/* fribee-ng */
#include "list.h"
#include "fs.h"

/**
 * supported filesystems by frisbee-ng (currently)
 */
extern struct file_system afs_fs;
extern struct file_system befs_fs;
extern struct file_system exfat_fs;
extern struct file_system ext2_fs;
extern struct file_system fat_fs;
extern struct file_system fng_fs;
extern struct file_system hfs_fs;
extern struct file_system hfsp_fs;
extern struct file_system hpfs_fs;
extern struct file_system jfs_fs;
extern struct file_system minix_fs;
extern struct file_system ntfs_fs;
extern struct file_system qnx4_fs;
extern struct file_system qnx6_fs;
extern struct file_system raw_fs;
extern struct file_system reiser_fs;
extern struct file_system xfs_fs;
extern struct file_system f2fs_fs;

LIST_HEAD(filesystems);

static void register_filesystem(struct file_system *fs)
{
	INIT_LIST_HEAD(&fs->list);
	list_add_tail(&fs->list, &filesystems);
}

void register_filesystems(void)
{
	register_filesystem(&ntfs_fs);
	register_filesystem(&afs_fs);
	register_filesystem(&xfs_fs);
	register_filesystem(&ext2_fs);
	register_filesystem(&jfs_fs);
	register_filesystem(&hfs_fs);
	register_filesystem(&hfsp_fs);
	register_filesystem(&befs_fs);
	register_filesystem(&hpfs_fs);
	register_filesystem(&reiser_fs);
	register_filesystem(&exfat_fs);
	register_filesystem(&minix_fs);
	register_filesystem(&raw_fs);

	/* these are not ready */
	//register_filesystem(&fat_fs);
	//register_filesystem(&qnx4_fs);
	//register_filesystem(&qnx6_fs);
	//register_filesystem(&fng_fs);
	//register_filesystem(&f2fs_fs);
}
