
/**
 * Copyright © 2005 Felix von Leitner
 * Copyright © 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/**
 * Trivial fixed-size ring buffer meant for writing to non-blocking sockets.
 * You send messages using ringbuf_append, which will return -1 if there
 * was no space for the whole message, not having appended anything to
 * the ring buffer at all.  This way you can detect non-responsive
 * connections.  When a socket signals writability, you call
 * ringbuf_write with the socket, which then does write or writev to
 * write the contents of the ring buffer and returns the number of bytes
 * written or -1 for temporary failure or -3 for hard failure.
 */

struct ringbuf {
	char *buf;
	size_t size, first, bytes;
};

void ringbuf_init(struct ringbuf *r, char *buf, size_t size);

int ringbuf_init_malloc(struct ringbuf *r, size_t size);

/**
 * returns -1 for "buffer full",
 *         0 for "ok",
 *         1 for "ok; buffer was empty before" (call io_wantwrite)
 */
ssize_t ringbuf_append(struct ringbuf *r, const char *s);

ssize_t ringbuf_appendbin(struct ringbuf *r, const char *s, size_t l);

ssize_t ringbuf_write(struct ringbuf *r, int s);

ssize_t ringbuf_read(struct ringbuf *r, int s);

size_t fmt_ringbuf(char *s, struct ringbuf *r);

int ringbuf_empty(struct ringbuf *r);

size_t ringbuf_spaceleft(struct ringbuf *r);

void ringbuf_free(struct ringbuf *r);
