
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* system */
#include <sys/types.h>

/* frisbee-ng */
#include "die.h"
#include "mem.h"
#include "shuffle.h"
#include "fmtX.h"

#include "haveinline.h"

static inline u64 prng_get64(void)
{
	u64 v;
	void *x = &v;

	fmt_random(x, 8);

	return v;
}

/**
 * swap to values of type size_t
 */
static void swap(size_t * a, size_t * b)
{
	register size_t x = *a;

	*a = *b;
	*b = x;
}

/**
 * get_shuffle - shuffle an array of n elements
 *
 * @n: number of elemetns, the shuffled array will have
 */
size_t *shuffle(size_t n)
{
	size_t *shuffle;
	size_t i, j;

	shuffle = mem_talloc(size_t, n);

	/* fill the array 0,1,2,3... */
	for (i = 0; i < n; i++)
		shuffle[i] = i;

	/**
	 * Fisher–Yates shuffle, see wikipedia
	 */
	for (j = n - 1; j > 0; j--) {
		i = prng_get64() % (j + 1);
		swap(&shuffle[j], &shuffle[i]);
	}

	return shuffle;
}
