
/**
 * Copyright © 2012 - 2014 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef M_SPLICE_H
#define M_SPLICE_H

#include "havesplice.h"

#ifdef HAVE_SPLICE

#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <endian.h>
#include <unistd.h>

#include "haveinline.h"
#include "defs.h"

#ifndef SPLICE_F_MOVE
#define SPLICE_F_MOVE		0x01
#define SPLICE_F_NONBLOCK	0x02
#define SPLICE_F_MORE		0x04
#define SPLICE_F_GIFT		0x08
#define SPLICE_F_UNMAP		0x10
#endif				/* SPLICE_F_MOVE */

/* we are wrapping these, for debugging when needed */
static inline s64 sys_splice(int fdin, s64 * off_in, int fdout, s64 * off_out,
			     u64 len, unsigned int flags)
{
	loff_t *offin = (loff_t *) off_in;
	loff_t *offout = (loff_t *) off_out;

	return splice(fdin, offin, fdout, offout, len, flags);
}

static inline s64 sys_tee(int fdin, int fdout, u64 len, unsigned int flags)
{
	return tee(fdin, fdout, len, flags);
}

/* on error, splice_loop() calls die_msg() */
extern s64 splice_loop(int fdin, s64 * off_in, int fdout, s64 * off_out,
		       u64 len, unsigned int flags);

/* on error, tee_loop() calls die_msg() */
extern s64 tee_loop(int fd_in, int fd_out, u64 len, unsigned int flags);

#endif				/* HAVE_SPLICE */
#endif				/* M_SPLICE_H */
