
/**
 * Copyright © 2010 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "havesplice.h"

#ifdef HAVE_SPLICE

#include "splice.h"
#include "die.h"
#include "wrapper.h"

s64 splice_loop(int fd_in, s64 * off_in, int fd_out, s64 * off_out, u64 len,
		unsigned int flags)
{
	u64 done = 0;

	do {
		s64 tlen = sys_splice(fd_in, off_in, fd_out, off_out,
				      len, flags);

		/**
		 * we catch no special errors!
		 * -> eg my 3.2.11 returns EAGAIN sometimes, I don't know why!
		 *    -> but waiting and trying later also causes EAGAIN AGAIN!
		 */
		if (tlen < 0)
			die_msg(111, "splice()");
		if (tlen == 0)
			return done;

		done += tlen;
	}
	while (done != len);

	return done;
}

#endif
