
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* system */
#include <pthread.h>

/* frisbee-ng */
#include "stats.h"
#include "defs.h"
#include "die.h"

void stats_init(stats_t * s)
{
	int r;

	r = pthread_mutex_init(&s->mutex, 0);
	if (unlikely(r != 0))
		die_pthread(r);

	/* get current time */
	gettimeofday(&s->t0, 0);

	s->accept = 0;
	s->read = 0;
	s->connect = 0;
	s->completed = 0;
	s->renew = 0;

	return;
}

void stats_deinit(stats_t * s)
{
	int r;

	r = pthread_mutex_destroy(&s->mutex);
	if (unlikely(r != 0))
		die_pthread(r);

	return;
}

void stats_update(stats_t * s, int what)
{
	int r;

	r = pthread_mutex_lock(&s->mutex);
	if (unlikely(r != 0))
		die_pthread(r);

	/* get current time */
	gettimeofday(&s->tc, 0);

	switch (what) {
	case EVENT_ACCEPT:
		s->accept++;
		break;
	case EVENT_READ:
		s->read++;
		break;
	case EVENT_CONNECT:
		s->connect++;
		break;
	case EVENT_COMPLETED:
		s->completed++;
		break;
	default:
		die_msg(111, "unknown statistic");
	}

	r = pthread_mutex_unlock(&s->mutex);
	if (unlikely(r != 0))
		die_pthread(r);

	return;
}
