
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef PROGRESS_H
#define PROGRESS_H

/* system */
#include <sys/types.h>
#include <sys/time.h>
#include <pthread.h>

#include "defs.h"

typedef enum {
	EVENT_ACCEPT,
	EVENT_READ,
	EVENT_CONNECT,
	EVENT_COMPLETED,
	EVENT_RENEW
} stats_events;

typedef struct {
	pthread_mutex_t mutex;	/* mutex, if used by multiple threads */
	struct timeval t0;	/* will be set by stats_init() */
	struct timeval tc;	/* will be set by stats_update() */

	u64 accept;
	u64 read;
	u64 connect;
	u64 completed;
	u64 renew;
} stats_t;

/* initialize structure for statistics */
extern void stats_init(stats_t * s);
extern void stats_deinit(stats_t * s);

/* increment counter */
extern void stats_update(stats_t * s, int event);

#endif
