
/**
 * Copyright © 2001 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef STRALLOCX_H
#define STRALLOCX_H

#include <stdarg.h>

#include "defs.h"
#include "stralloc.h"

/**
 * some extensions to the standard stralloc routines
 */

extern int stralloc_cat_uint(stralloc * sa, unsigned int in);

extern int stralloc_cat_uuid(stralloc * sa, char in[16]);

/**
 * "12345678" -> "12.345.678" (dot = '.')
 */
extern int stralloc_cat_dotted(stralloc * sa, const unsigned long long in,
			       const char dot);

/**
 * "foo" -> "foo  "
 */
extern int stralloc_cat_padleft(stralloc * sa, size_t srclen, size_t padlen,
				size_t maxlen, char with);

/**
 * "foo" -> "  foo"
 */
extern int stralloc_cat_padright(stralloc * sa, const char *src, size_t srclen,
				 size_t padlen, size_t maxlen, char with);

/**
 * good for crypto stuff:
 */
extern int stralloc_cat_zero(stralloc * sa, int n);
extern int stralloc_cat_random(stralloc * sa, int n);

/**
 * main stralloc_cat_() functions for stralloc_fmt()
 */
extern int stralloc_fmt(stralloc * sa, const char *fmt, va_list ap);
extern int stralloc_print(stralloc * sa, const char *fmt, ...);

extern int stralloc_cat_human(stralloc * sa, unsigned long long in);
extern int stralloc_cat_humank(stralloc * sa, unsigned long long in);

extern int stralloc_cat_s64pw(stralloc * sa, long long i, int pad, char with);

extern int stralloc_cat_x64pw(stralloc * sa, unsigned long long i, int pad,
			      char with);

extern int stralloc_cat_u64pw(stralloc * sa, unsigned long long i, int pad,
			      char with);

/**
 * cpu "123" -> little endian "123"
 */
extern int stralloc_cat_le32(stralloc * sa, u32 in);
extern int stralloc_cat_le64(stralloc * sa, u64 in);

#endif				/* STRALLOCX_H */
