
/**
 * Copyright © 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* libowfat */
#include "str.h"
#include "scan.h"
#include "stralloc.h"
#include "openreadclose.h"

/* frisbee-ng */
#include "die.h"
#include "sysstat.h"
#include "defs.h"
#include "mem.h"
#include "die.h"
#include "term.h"

/**
 * should work on 2.6 kernels and newer... /TR
 */

static void read_procfile(char *file, stralloc * sa)
{
	int r;
	unsigned i;

	stralloc_init(sa);
	r = openreadclose(file, sa, 1024 * 10);

	if (r != 1)
		die_msg(2, "Can't open ", file, " :-(");

	/* split the stralloc @ each newline, this is used in later parsing */
	for (i = 0; i < sa->len; i++)
		if (sa->s[i] == '\n')
			sa->s[i] = 0;

	return;
}

/* used a lot ... */
static const char *tabspace = "\t ";

void sysstat_cpu(cpustat_t * c)
{
	unsigned int j;
	stralloc cpu;

	if (c->name.len == 0)
		return;
	read_procfile("/proc/stat", &cpu);

#if 0
	typedef struct {
		stralloc name;	/* "cpu  " */
		u64 user;	/* field 1 */
		u64 nice;	/* field 2 */
		u64 system;	/* field 3 */
		u64 idle;	/* field 4 */
		u64 iowait;	/* field 5 */
	} cpustat_t;

 /proc / stat:
	user nice system idle iowait irq softirq steal guest guestnice cpu
	    2279990 95902 1299855 18868648 475786 2297 89759 0 0 0
#endif
	    for (j = 0; j < cpu.len; j += str_len(cpu.s + j) + 1) {
		long long unsigned u;
		char *line = cpu.s + j;
		char *p = line + 5;

		/* total cpu stats, not cpu0 .. cpuN */
		if (!str_start(line, c->name.s))
			continue;
		c->time = time(0);

		/* term_s(line); term_flush(); */
		p += scan_ulonglong(p, &u);
		c->user = u;
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);
		c->nice = u;
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);
		c->system = u;
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);
		c->idle = u;
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);
		c->iowait = u;
	}

	stralloc_free(&cpu);
	return;
}

void sysstat_net(netstat_t * n)
{
	unsigned int j;
	stralloc net;

	if (n->name.len == 0)
		return;
	read_procfile("/proc/net/dev", &net);

#if 0
	typedef struct {
		stralloc name;	/* "eth0" */
		u64 in_bytes;	/* field 1 */
		u64 in_packets;	/* field 2 */
		u64 in_errors;	/* field 3 */
		u64 in_dropped;	/* field 4 */
		u64 in_mcast;	/* field 8 */

		u64 out_bytes;	/* field 9 */
		u64 out_packets;	/* field 10 */
		u64 out_errors;	/* field 11 */
		u64 out_dropped;	/* field 12 */
		u64 out_colls;	/* field 14 */
	} netstat_t;

 /proc / net / dev:
 Interface | rx bytes packets errs drop fifo frame compressed multicast | tx bytes packets errs drop fifo colls carrier compressed lo: 9909254 103698 0 0 0 0 0 0 9909254 103698 0 0 0 0 0 0 vboxnet0: 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 wlan0: 5887948915 4895905 0 0 0 0 0 0 789390794 3409494 0 0 0 0 0 0 eth0:63449698 149830 0 10 0 0 0 24 11757260 100947 0 0 0 0 0
	    0
#endif
	    for (j = 0; j < net.len; j += str_len(net.s + j) + 1) {
		long long unsigned u;
		char *p = net.s + j;

		/* bis an das interface */
		p += scan_charsetnskip(p, tabspace, 20);
		if (!str_start(p, n->name.s))
			continue;
		//if (!str_start(p, "eth0:")) continue;
		n->time = time(0);

		/* interface inkl. ":" überspringen */
		p += scan_noncharsetnskip(p, tabspace, 20);
		/* bytes danach überspringen */
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 1 */
		p += scan_ulonglong(p, &u);
		n->in_bytes = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 2 */
		p += scan_ulonglong(p, &u);
		n->in_packets = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 3 */
		p += scan_ulonglong(p, &u);
		n->in_errors = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 4 */
		p += scan_ulonglong(p, &u);
		n->in_dropped = u;
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);	/* 5 */
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);	/* 6 */
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);	/* 7 */
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 8 */
		p += scan_ulonglong(p, &u);
		n->in_mcast = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 9 */
		p += scan_ulonglong(p, &u);
		n->out_bytes = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 10 */
		p += scan_ulonglong(p, &u);
		n->out_packets = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 11 */
		p += scan_ulonglong(p, &u);
		n->out_errors = u;
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 12 */
		p += scan_ulonglong(p, &u);
		n->out_dropped = u;
		p += scan_charsetnskip(p, tabspace, 20);
		p += scan_ulonglong(p, &u);	/* 13 */
		p += scan_charsetnskip(p, tabspace, 20);

		/* field 14 */
		p += scan_ulonglong(p, &u);
		n->out_colls = u;
		p += scan_charsetnskip(p, tabspace, 20);

	}

	stralloc_free(&net);
	return;
}

void sysstat_disk(diskstat_t * d)
{
	unsigned int j;
	stralloc disk;

	if (d->name.len == 0)
		return;
	read_procfile("/proc/diskstats", &disk);

#if 0
	typedef struct {
		stralloc name;	/* "sda" */
		u64 rd_ios;	/*  1) # of reads completed */
		u64 rd_sectors;	/*  3) # of sectors read */
		u64 rd_ticks;	/*  4) # of milliseconds spent reading */

		u64 wr_ios;	/*  5) # of writes completed */
		u64 wr_sectors;	/*  7) # of sectors written */
		u64 wr_ticks;	/*  8) # of milliseconds spent writing */

		u64 rdwr_ticks;	/* 10) # of milliseconds spent doing I/Os */
		u64 aveq;	/* 11) weighted # of milliseconds spent doing I/Os */
	} diskstat_t;

 /proc / diskstats:
	8 0 sda 724854 21675 48125705 6590290 1216937 146658 108883688 134422503
	    0 4638763 141043113 8 0 sda 735380 21682 48603937 6693040 1238792
	    150767 109832336 136301170 0 4734186 143024516
#endif
	    for (j = 0; j < disk.len; j += str_len(disk.s + j) + 1) {
		long long unsigned u;
		char *p = disk.s + j + 13;

		//term_print("\n%s", p); term_flush();
		if (!str_start(p, d->name.s))
			continue;
		d->time = time(0);

		/* disk überspringen */
		p += scan_noncharsetnskip(p, tabspace, 20);
		p += scan_charsetnskip(p, tabspace, 10);

		/* 1) # of reads completed */
		p += scan_ulonglong(p, &u);
		d->rd_ios = u;
		p += scan_charsetnskip(p, tabspace, 10);
		p += scan_ulonglong(p, &u);	/* 2 */
		p += scan_charsetnskip(p, tabspace, 10);

		/* 3) # of sectors read */
		p += scan_ulonglong(p, &u);
		d->rd_sectors = u;
		p += scan_charsetnskip(p, tabspace, 10);

		/* 4) # of milliseconds spent reading */
		p += scan_ulonglong(p, &u);
		d->rd_ticks = u;
		p += scan_charsetnskip(p, tabspace, 10);

		/* 5) # of writes completed */
		p += scan_ulonglong(p, &u);
		d->wr_ios = u;
		p += scan_charsetnskip(p, tabspace, 10);
		p += scan_ulonglong(p, &u);	/* 6 */
		p += scan_charsetnskip(p, tabspace, 10);

		/* 7) # of sectors written */
		p += scan_ulonglong(p, &u);
		d->wr_sectors = u;
		p += scan_charsetnskip(p, tabspace, 10);

		/* 8) # of milliseconds spent writing */
		p += scan_ulonglong(p, &u);
		d->wr_ticks = u;
		p += scan_charsetnskip(p, tabspace, 10);
		p += scan_ulonglong(p, &u);	/* 9 */
		p += scan_charsetnskip(p, tabspace, 10);

		/* 10) # of milliseconds spent doing I/Os */
		p += scan_ulonglong(p, &u);
		d->rdwr_ticks = u;
		p += scan_charsetnskip(p, tabspace, 10);

		/* 11) weighted # of milliseconds spent doing I/Os */
		p += scan_ulonglong(p, &u);
		d->aveq = u;
		p += scan_charsetnskip(p, tabspace, 10);
	}

	stralloc_free(&disk);

	return;
}

sysstat_t *sysstat_init(char *cpu, char *netdev, char *diskdev)
{
	sysstat_t *s = mem_tzalloc(sysstat_t, 1);

	stralloc_init(&s->cpu.name);
	stralloc_init(&s->net.name);
	stralloc_init(&s->disk.name);

	if (!stralloc_copys(&s->cpu.name, cpu))
		die_nomem();
	if (!stralloc_copys(&s->net.name, netdev))
		die_nomem();
	if (!stralloc_copys(&s->disk.name, diskdev))
		die_nomem();

	if (!stralloc_0(&s->cpu.name))
		die_nomem();
	if (!stralloc_0(&s->net.name))
		die_nomem();
	if (!stralloc_0(&s->disk.name))
		die_nomem();

	return s;
}

void sysstat_deinit(sysstat_t * s)
{
	stralloc_free(&s->cpu.name);
	stralloc_free(&s->net.name);
	stralloc_free(&s->disk.name);
	mem_free(s);
}

void sysstat(sysstat_t * s)
{
	time_t ct = time(0);
	double total = 0;

	/* < 1 second interval is too fast */
	if (ct == s->time)
		return;

	s->cpu_last = s->cpu;
	s->net_last = s->net;
	s->disk_last = s->disk;
	s->time_last = s->time;

	s->time = ct;
	sysstat_cpu(&s->cpu);
	sysstat_net(&s->net);
	sysstat_disk(&s->disk);

	/* first call */
	if (s->time_last == 0) {
		s->cpu_first = s->cpu;
		s->net_first = s->net;
		s->disk_first = s->disk;
		s->time_first = s->time;
		return;
	}

	/* calculate diff to last one */
	s->cpu_diff.user = s->cpu.user - s->cpu_last.user;
	s->cpu_diff.system = s->cpu.system - s->cpu_last.system;
	s->cpu_diff.idle = s->cpu.idle - s->cpu_last.idle;
	s->cpu_diff.iowait = s->cpu.iowait - s->cpu_last.iowait;
	total = s->cpu_diff.user + s->cpu_diff.system;
	total += s->cpu_diff.idle + s->cpu_diff.iowait;
	total /= 100.0;
	s->cpu_diff.user /= total;
	s->cpu_diff.system /= total;
	s->cpu_diff.idle /= total;
	s->cpu_diff.iowait /= total;

	/* calculate diff to first one */
	s->cpu_total.user = s->cpu.user - s->cpu_first.user;
	s->cpu_total.system = s->cpu.system - s->cpu_first.system;
	s->cpu_total.idle = s->cpu.idle - s->cpu_first.idle;
	s->cpu_total.iowait = s->cpu.iowait - s->cpu_first.iowait;
	total = s->cpu_total.user + s->cpu_total.system;
	total += s->cpu_total.idle + s->cpu_total.iowait;
	total /= 100.0;
	s->cpu_total.user /= total;
	s->cpu_total.system /= total;
	s->cpu_total.idle /= total;
	s->cpu_total.iowait /= total;

	//s->netd = s->net0 - s->net1;
	s->net_diff.time = s->net.time - s->net_last.time;
	s->net_diff.in_bytes = s->net.in_bytes - s->net_last.in_bytes;
	s->net_diff.in_packets = s->net.in_packets - s->net_last.in_packets;

// u64 rd_ios;      /*  1) # of reads completed */
// u64 rd_sectors;  /*  3) # of sectors read */
// u64 rd_ticks;    /*  4) # of milliseconds spent reading */
// 
// u64 wr_ios;      /*  5) # of writes completed */
// u64 wr_sectors;  /*  7) # of sectors written */
// u64 wr_ticks;    /*  8) # of milliseconds spent writing */
// 
// u64 rdwr_ticks;  /* 10) # of milliseconds spent doing I/Os */
// u64 aveq;        /* 11) weighted # of milliseconds spent doing I/Os */

	s->time_diff = s->time - s->time_last;
	s->time_total = s->time - s->time_first;
}
