
/**
 * Copyright © 2009 - 2012 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef SYSSTAT_H
#define SYSSTAT_H

/* system */
#include <sys/types.h>
#include <sys/time.h>
#include <pthread.h>

/* libowfat */
#include "stralloc.h"

/* frisbee */
#include "defs.h"

/**
 * das hier ist recht umfangreich, aber dient nur der vollständigen kontrolle
 * von frisbee, man bekommt da sicher auch gut diverse fehler heraus oder kann
 * was optimieren ;)
 * /TR 2012-03-04
 */

typedef struct {
	stralloc name;		/* IN: "cpu " */
	time_t time;		/* data timestamp */

	u64 user;		/* field 1 */
	u64 nice;		/* field 2 */
	u64 system;		/* field 3 */
	u64 idle;		/* field 4 */
	u64 iowait;		/* field 5 */
} cpustat_t;

typedef struct {
	stralloc name;		/* IN: "eth0:" */
	time_t time;		/* data timestamp */

	u64 in_bytes;		/* field 1 */
	u64 in_packets;		/* field 2 */
	u64 in_errors;		/* field 3 */
	u64 in_dropped;		/* field 4 */
	u64 in_mcast;		/* field 9 */

	u64 out_bytes;		/* field 10 */
	u64 out_packets;	/* field 11 */
	u64 out_errors;		/* field 12 */
	u64 out_dropped;	/* field 13 */
	u64 out_colls;		/* field 15 */
} netstat_t;

typedef struct {
	stralloc name;		/* IN: "sda" */
	time_t time;		/* data timestamp */

	u64 rd_ios;		/*  1) # of reads completed */
	u64 rd_sectors;		/*  3) # of sectors read */
	u64 rd_ticks;		/*  4) # of milliseconds spent reading */

	u64 wr_ios;		/*  5) # of writes completed */
	u64 wr_sectors;		/*  7) # of sectors written */
	u64 wr_ticks;		/*  8) # of milliseconds spent writing */

	u64 rdwr_ticks;		/* 10) # of milliseconds spent doing I/Os */
	u64 aveq;		/* 11) weighted # of milliseconds spent doing I/Os */
} diskstat_t;

typedef struct {
	time_t time;		/* current */
	time_t time_last;	/* last one */
	time_t time_first;	/* first one */
	time_t time_diff;	/* diff to last one */
	time_t time_total;	/* diff since first call */

	cpustat_t cpu;		/* current */
	cpustat_t cpu_last;	/* last */
	cpustat_t cpu_first;	/* first */
	cpustat_t cpu_diff;	/* diff */
	cpustat_t cpu_total;	/* total */

	netstat_t net;
	netstat_t net_last;
	netstat_t net_first;
	netstat_t net_diff;
	netstat_t net_total;

	diskstat_t disk;
	diskstat_t disk_last;
	diskstat_t disk_first;
	diskstat_t disk_diff;
	diskstat_t disk_total;
} sysstat_t;

void sysstat_cpu(cpustat_t * c);
void sysstat_net(netstat_t * n);
void sysstat_disk(diskstat_t * d);

sysstat_t *sysstat_init(char *cpu, char *netdev, char *diskdev);
void sysstat_deinit(sysstat_t * s);
void sysstat(sysstat_t * s);

#endif
