
/**
 * Copyright © 2008-2010 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 13
 */

/* system */
#include <pthread.h>

#include "strallocX.h"
#include "term.h"
#include "tty.h"

/* terminal output buffer with mutex */
static pthread_mutex_t term_mutex = PTHREAD_MUTEX_INITIALIZER;
static stralloc sa_term;

/* terminal resolution */
int term_maxx, term_maxy;
static int term_init_done = 0;

/* various terminal standard strings */
char *term_str_norm = T_NORMAL;
char *term_str_bold = T_BOLD;
char *term_str_init = T_INIT;
char *term_str_quit = T_QUIT;
char *term_str_clear = T_CLEAR;

/* left top status line ... */
stralloc term_left_status_top;

/**
 * called once @ begin
 */
void term_init(void)
{
	if (term_init_done) {
		return;
	}

	stralloc_init(&sa_term);
	stralloc_init(&term_left_status_top);
	term_init_done = 1;

	if (!is_tty(1)) {
		return;
	}

	/* we may use fancy things, when on tty */
	term_s(term_str_init);
	term_flush();

	return;
}

/**
 * called once @ end
 */
void term_quit(void)
{
	if (!term_init_done) {
		return;
	}

	term_flush();
	term_init_done = 0;

	if (!is_tty(1)) {
		return;
	}

	/* flush old buffer */
	term_s(term_str_quit);
	term_s(term_str_norm);
	term_flush();

	return;
}

void term_ch(const char add)
{
	pthread_mutex_lock(&term_mutex);
	if (!stralloc_catb(&sa_term, &add, 1))
		die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_s(const char *add)
{
	pthread_mutex_lock(&term_mutex);
	if (!stralloc_cats(&sa_term, add))
		die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_sa(stralloc * add)
{
	pthread_mutex_lock(&term_mutex);
	if (!stralloc_cat(&sa_term, add))
		die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_uint(u32 add)
{
	pthread_mutex_lock(&term_mutex);
	if (!stralloc_cat_uint(&sa_term, add))
		die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_flush(void)
{
	pthread_mutex_lock(&term_mutex);
	buffer_putsaflush(buffer_1, &sa_term);
	if (!stralloc_copys(&sa_term, ""))
		die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_gotoxy(int x, int y)
{
	pthread_mutex_lock(&term_mutex);
	if (!stralloc_cats(&sa_term, "\e["))
		die_nomem();
	if (!stralloc_cat_uint(&sa_term, y))
		die_nomem();
	if (!stralloc_cats(&sa_term, ";"))
		die_nomem();
	if (!stralloc_cat_uint(&sa_term, x))
		die_nomem();
	if (!stralloc_cats(&sa_term, "H"))
		die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_hline(int i, const char *s)
{
	pthread_mutex_lock(&term_mutex);
	while (--i)
		if (!stralloc_cats(&sa_term, s))
			die_nomem();
	pthread_mutex_unlock(&term_mutex);

	return;
}

void term_printkey(unsigned char *key, int len, char *msg)
{
	int i;

	term_print("%s 0x", msg);
	for (i = 0; i < len; i++)
		term_print("%02x", key[i] & 0xff);
	term_print(" (len=%d)\n", len);
	term_flush();
}
