
/**
 * Copyright © 2008-2009 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef M_TERM_H
#define M_TERM_H

/* libowfat */
#include "stralloc.h"
#include "buffer.h"

#include "defs.h"
#include "die.h"

/*
 * ESC seqences, you may change some of them ...
 * (these are vt100, xterm, linux, xterm-color compatible)
 *
 * style:  00=none 01=bold 04=underscore 05=blink 07=reverse 08=concealed
 * fg: 30=black 31=red 32=green 33=yellow 34=blue 35=magenta 36=cyan 37=grey
 * bg: 40=black 41=red 42=green 43=yellow 44=blue 45=magenta 46=cyan 47=grey
 */

#define T_NORMAL      "\e[0m"	/* normal mode */
#define T_BOLD        "\e[1m"	/* bold mode */
#define T_HOME        "\e[H"	/* goto 0,0 */
#define T_CLEAR       "\e[2J"	/* clear screen */
#define T_INIT        "\e[?25l\e(B\e)0"	/* init stuff, hide also mouse */
#define T_QUIT        "\e[?25h"	/* quit stuff, unhide mouse */

#define C_BOX         "\e[0;34m"	/* blue   - boxdrawing */
#define C_PACMAN      "\e[1;33m"	/* yellow - the eating pacman ;-) */
#define C_CLUSTER     "\e[0;37m"	/* grey   - cluster */
#define C_NORMAL      "\e[0;37m"	/* grey   - normal */
#define C_STATUS      "\e[0;31m"	/* red    - status lines */
#define C_DESCRIPTION "\e[1;33m"	/* yellow - descriptions */
#define C_HEADLINE    "\e[0;33m"	/* brown  - version, headlines */

#define C_BLOCK_FREE    " "	/*     0% */
#define C_BLOCK_TO33    "."	/*  1-33% */
#define C_BLOCK_TO66    "o"	/* 34-66% */
#define C_BLOCK_TO99    "O"	/* 67-99% */

/* is the maxima of the left descriptions! (term.c) */
extern int term_maxx;
extern int term_maxy;

/* various terminal standard strings (term.c) */
extern char *term_str_clear;
extern char *term_str_norm;
extern char *term_str_bold;
extern char *term_str_init;
extern char *term_str_quit;

/* init / quit strings for terminal (term.c) */
extern void term_init(void);	/* big init ;) */
extern void term_quit(void);	/* big exit ;) */
extern void term_ch(const char add);
extern void term_s(const char *add);
extern void term_sa(stralloc * add);
extern void term_uint(u32 add);

/* for debugging */
extern void term_printkey(unsigned char *key, int len, char *msg);

/* simple interactive screen.... */
extern void term_resize(void);

/* set cursor to x,y */
extern void term_gotoxy(int x, int y);

/* draw a line with length i and character ch */
extern void term_hline(int i, const char *ch);

/* draw a box */
extern void term_box(int x1, int y1, int x2, int y2);
extern void term_adhoc_client(void);
extern void term_adhoc_server(void);

/* flush stralloc to buffer_1 (stdout) */
extern void term_flush(void);

/**
 * term_clustermap - show / update the clustermap
 * @bitmap:   allocated bitmap of filesystem
 * @blocks:   total number of blocks in filesystem
 * @bcurrent: current working block (fng-image / fng-clone)
 */
struct term_clustermap {

	/* within these values the termmap will be printed */
	int x1, x2, y1, y2;
	u8 *bitmap;
	u64 blocks;

	/* these values are calculated by term_clustermap_init */
	u32 blocks_per_unit;
	int lenx, leny;
	u64 needed_units;
};

void term_clustermap_init(struct term_clustermap *tcmap, u8 * bitmap,
			  u64 blocks, int x1, int x2, int y1, int y2);
void term_clustermap(struct term_clustermap *tcmap, u64 bcurrent);

extern void term_print(const char *fmt, ...);
extern void term_printflush(const char *fmt, ...);

/* functions for the main window handling */
#define term_main(l,r) term_main_colorS(0,l,0,r)
#define term_main_color(cl,l,cr,r) term_main_color_doit(0,cl,l,cr,r)	/* permanent */
#define term_main_colorS(cl,l,cr,r) term_main_color_doit(1,cl,l,cr,r)	/* scrolling */
extern void term_main_color_doit(int type, int cleft, stralloc * left,
				 int cright, stralloc * right);
extern void term_main_clear(void);
extern void term_main_reset(void);
extern void term_main_scroll(int x);

/* used by fng-clone */
extern void term_fng_clone(void);

extern void term_progress(int x_start, int x_end, int percent);

/* left status bar on top... */
extern stralloc term_left_status_top;

#endif
