
/**
 * Copyright © 2008-2009 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 13
 */

#include "str.h"

#include "main.h"
#include "term.h"

/**
 * utf8 art ;)
 * ╔══╦══╗  ┌──┬──┐  ╭──┬──╮  ╭──┬──╮  ┏━━┳━━┓  ┎┒┏┑   ╷  ╻ ┏┯┓ ┌┰┐    ▊ ╱╲╱╲╳╳╳
 * ║┌─╨─┐║  │╔═╧═╗│  │╒═╪═╕│  │╓─╁─╖│  ┃┌─╂─┐┃  ┗╃╄┙  ╶┼╴╺╋╸┠┼┨ ┝╋┥    ▋ ╲╱╲╱╳╳╳
 * ║│╲ ╱│║  │║   ║│  ││ │ ││  │║ ┃ ║│  ┃│ ╿ │┃  ┍╅╆┓   ╵  ╹ ┗┷┛ └┸┘    ▌ ╱╲╱╲╳╳╳
 * ╠╡ ╳ ╞╣  ├╢   ╟┤  ├┼─┼─┼┤  ├╫─╂─╫┤  ┣┿╾┼╼┿┫  ┕┛┖┚     ┌┄┄┐ ╎ ┏┅┅┓ ┋ ▍ ╲╱╲╱╳╳╳
 * ║│╱ ╲│║  │║   ║│  ││ │ ││  │║ ┃ ║│  ┃│ ╽ │┃  ░░▒▒▓▓██ ┊  ┆ ╎ ╏  ┇ ┋ ▎
 * ║└─╥─┘║  │╚═╤═╝│  │╘═╪═╛│  │╙─╀─╜│  ┃└─╂─┘┃  ░░▒▒▓▓██ ┊  ┆ ╎ ╏  ┇ ┋ ▏
 * ╚══╩══╝  └──┴──┘  ╰──┴──╯  ╰──┴──╯  ┗━━┻━━┛  ▗▄▖▛▀▜   └╌╌┘ ╎ ┗╍╍┛ ┋  ▁▂▃▄▅▆▇█
 */

#if 0

/* utf8 */
#define AC_TL      "┌"	/* top left */
#define AC_TR      "┐"	/* top right */
#define AC_BL      "└"	/* bottom left */
#define AC_BR      "┘"	/* bottom right */
#define AC_MT      "┬"	/* top 2middle */
#define AC_MB      "┴"	/* bottom 2middle */
#define AC_ML      "├"	/* left 2middle */
#define AC_MR      "┤"	/* right 2middle */
#define AC_MM      "┼"	/* middle + */
#define AC_HL      "─"	/* hline - */
#define AC_VL      "│"	/* vline | */

#else

/* non utf8 */
#define AC_TL      "\016l\017"	/* top left */
#define AC_TR      "\016k\017"	/* top right */
#define AC_BL      "\016m\017"	/* bottom left */
#define AC_BR      "\016j\017"	/* bottom right */
#define AC_MT      "\016w\017"	/* top 2middle */
#define AC_MB      "\016v\017"	/* bottom 2middle */
#define AC_ML      "\016t\017"	/* left 2middle */
#define AC_MR      "\016u\017"	/* right 2middle */
#define AC_MM      "\016n\017"	/* middle + */
#define AC_HL      "\016q\017"	/* hline - */
#define AC_VL      "\016x\017"	/* vline | */
#endif

void term_box(int x1, int y1, int x2, int y2)
{
	register int i;

	term_gotoxy(x1, y1);
	term_s(C_BOX);		/* coloring */
	term_s(AC_TL);
	term_hline(x2 - x1, AC_HL);
	term_s(AC_TR);
	for (i = 1; i != y2 - 1; ++i) {
		term_gotoxy(x1, y1 + i);
		term_s(AC_VL);
		term_gotoxy(x2, y1 + i);
		term_s(AC_VL);
	}
	term_gotoxy(x1, y2);
	term_s(AC_BL);
	term_hline(x2 - x1, AC_HL);
	term_s(AC_BR);

	return;
}

/**
 *  01 ┌────────────────────────────────────────────────────────────────────────────┐
 *  02 │ /dev/sda -> /dev/sdc                            fng-clone b282.a2bf2dd.v01 │
 *  03 ├────────────────────────────────────────────────────────────────────────────┤
 *  04 │cOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOooooooo.................................│
 *  05 │...............oooooooooooooooooooooOOOOOOOOoOOOOOOOoooooooo................│
 *  06 │OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOo│
 *  07 │ooooooooooooooooooooooo................................ooooo................│
 *  08 │............................................................................│
 *  09 │............................................................................│
 *  10 │............................................................................│
 *  11 │............................................................................│
 *  12 │............................................................................│
 *  13 │............................................................................│
 *  14 │OOOOO...........OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOoooooooooooOOOooo.......│
 *  15 │...............oooooooo....................................ooooooooooooooooo│
 *  16 │ooooooooooooooooooooooooooooOOO.............................................│
 *  17 │...............................................                             │ -9
 *  18 │                                                       1455 blocks per unit │ -8
 *  19 ├────────────────────────────────────────────────────────────────────────────┤ -7
 *  20 │ running time: 1:23:33, eta: 30:54:15                                       │ -6
 *  21 │ data rate: xxxx.yMiB/s  x778.2kiB/xx28.9GiB                                │ -5
 *  22 │                                                                            │ -4
 *  23 │ pxx/xx: [----------------------Coooooooooooooooooooooooooooooooooooo] 100% │ -3
 *  24 │ disk:   [-----------Cooooooooooooooooooooooooooooooooooooooooooooooo] 100% │ -2
 *  25 └────────────────────────────────────────────────────────────────────────────┘ -1
 *
 *  23 │ pxx/xx: ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ 100% │
 *  24 │ disk:   ▓▓▓▓▓▓▓▓▓▓▓▓▓░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ 100% │
 *
 *  23 │ pxx/xx: [======================>                                    ] 100% │
 *  24 │ disk:   [===========>                                               ] 100% │
 *
 * - wird auch für fng-image und fng-restore verwendet
 */

void term_fng_clone(void)
{
	int x;

	term_s(term_str_clear);
	term_box(1, 1, term_maxx, term_maxy - 1);

	term_gotoxy(1, 3);
	term_s(AC_ML);
	term_hline(term_maxx - 1, AC_HL);
	term_s(AC_MR);

	term_gotoxy(1, term_maxy - 7);
	term_s(AC_ML);
	term_hline(term_maxx - 1, AC_HL);
	term_s(AC_MR);

	term_s(term_str_norm);
	term_gotoxy(3, 2);
	term_sa(&term_left_status_top);

	x = term_maxx;
	x -= str_len(fng_name) + 1;
	x -= str_len(fng_version) + 1;
	term_gotoxy(x, 2);
	term_s(fng_name);
	term_s(" ");
	term_s(fng_version);

	return;
}

/*

server ip6:
┌─────────────────────────────────────────────────┬────────────────────────────┐
│ Disk: /dev/sda, 4 partitions                    │ frisbee-ng 0.1 SERVER mode │
│ IPv6: fe80::21d:e0ff:fe59:b88d via eth0         └────────────────────────────┤
│ Send: 123456789012  Rcvd: 12345678912            disk: 123 MiB/s   cpu: xxx% │
├──────────────────────────────────────────────────────────────────────────────┤
│ #  mac                ip              rx          tx         err             │
│ 1  00:1d:e1:59:b8:8d  192.168.100.21  1234567890  1234567890 1234567890      │
│ 2  00:1d:e2:59:b8:8d  192.168.100.22                                         │
│ 3  00:1d:e3:59:b8:8d  192.168.100.23                                         │
│ 4  00:1d:e4:59:b8:8d  192.168.100.24                                         │
│ 5  00:1d:e5:59:b8:8d  192.168.100.25                                         │
│ 6  00:1d:e6:59:b8:8d  192.168.100.26                                         │
│ 7  00:1d:e7:59:b8:8d  192.168.100.27                                         │
│ 8  00:1d:e8:59:b8:8d  192.168.100.28                                         │
│                                                                              │
├──────────────────────────────────────────────────────────────────────────────┤
│                                                                              │
└──────────────────────────────────────────────────────────────────────────────┘

┌─────────────────────────────────────────────────┬────────────────────────────┐
│ Disk: /dev/sda, 4 partitions                    │ frisbee-ng 0.1 CLIENT mode │
│ IPv4: 192.168.100.23 via eth0                   └────────────────────────────┤
│ Send: 123456789012  Rcvd: 12345678912            disk: 123 MiB/s   cpu: xxx% │
├──────────────────────────────────────────────────────────────────────────────┤
│ #  mac                ip              rx          tx         err             │
│ 1  00:1d:e1:59:b8:8d  192.168.100.21  1234567890  1234567890 1234567890      │
│ 2  00:1d:e2:59:b8:8d  192.168.100.22                                         │
│ 3  00:1d:e3:59:b8:8d  192.168.100.23                                         │
│ 4  00:1d:e4:59:b8:8d  192.168.100.24                                         │
│ 5  00:1d:e5:59:b8:8d  192.168.100.25                                         │
│ 6  00:1d:e6:59:b8:8d  192.168.100.26                                         │
│ 7  00:1d:e7:59:b8:8d  192.168.100.27                                         │
│ 8  00:1d:e8:59:b8:8d  192.168.100.28                                         │
│                                                                              │
├──────────────────────────────────────────────────────────────────────────────┤
│                                                                              │
└──────────────────────────────────────────────────────────────────────────────┘
*/

void term_adhoc_server(void)
{
	int y;

	term_s(C_BOX);		/* box colors now... */
	term_gotoxy(1, 1);
	term_s(AC_TL);
	term_hline(term_maxx - 1, AC_HL);
	term_s(AC_TR);

	term_gotoxy(1, 2);
	term_s(AC_VL);
	term_gotoxy(term_maxx, 2);
	term_s(AC_VL);

	term_gotoxy(1, 3);
	term_s(AC_VL);
	term_gotoxy(term_maxx - 30, 1);
	term_s(AC_MT);
	term_gotoxy(term_maxx - 30, 2);
	term_s(AC_VL);
	term_gotoxy(term_maxx - 30, 3);
	term_s(AC_BL);
	term_hline(30, AC_HL);
	term_s(AC_MR);

	for (y = 4; y != term_maxy; ++y) {
		term_gotoxy(1, y);
		term_s(AC_VL);
		term_gotoxy(term_maxx, y);
		term_s(AC_VL);
	}

	term_gotoxy(1, term_maxy);
	term_s(AC_BL);
	term_hline(term_maxx - 1, AC_HL);
	term_s(AC_BR);

	term_gotoxy(1, term_maxy - 2);
	term_s(AC_ML);
	term_hline(term_maxx - 1, AC_HL);
	term_s(AC_MR);

	term_gotoxy(1, 6);
	term_s(AC_ML);
	term_hline(term_maxx - 1, AC_HL);
	term_s(AC_MR);

	return;
}

/*
ad-hoc client:
┌─────────────────────────────────────────────────┬────────────────────────────┐
│ Disk: /dev/sda (54GiB)                          │ frisbee-ng 0.1 ad-hoc mode │
│ Server: 192.168.100.23 via eth0                 └────────────────────────────┤
│ Send: 123456789012  Rcvd: 12345678912            disk: 123 MiB/s   cpu: xxx% │
├──────────────────────────────────────────────────────────────────────────────┤
│                                                                              │
│ status of packets: .....                                                     │
│                                                                              │
│                                                                              │
│                                                                              │
│                                                                              │
│                                                                              │
│                                                                              │
│                                                                              │
│                                                                              │
├──────────────────────────────────────────────────────────────────────────────┤
│                                                                              │
└──────────────────────────────────────────────────────────────────────────────┘
*/
void term_adhoc_client(void)
{
	term_adhoc_server();	/* by now .... */

	return;
}
