
/**
 * Copyright © 2008 - 2014 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 13
 */

#include "bitmap.h"
#include "term.h"
#include "tty.h"
#include "strallocX.h"

/**
 *
 * 1920x1080 -> 240x67
 * 240*8 -> 1092
 * 67*16 -> 1072 (1080/16 -> 67)
 *
 * 800x600 -> 100x37
 */

/**
 * map: current position @ bitmap
 * perdot: blocks per unit
 */
static void term_blockmap_showunit(u8 * map, u64 block, u32 blocks_per_unit)
{
	u32 count = 0, c;
	u8 *start = map + ((block / 8) * blocks_per_unit);

	for (c = 0; c < blocks_per_unit; c++) {
		if (bitmap_get(start, c))
			count++;
	}

	if (count == 0) {
		term_s(".");
	} else if (count == blocks_per_unit) {
		term_s("O");
	} else {
		term_s("o");
	}
}

/**
 * init a new clustermap structure... must be done before calling term_clustermap()
 */
void term_clustermap_init(struct term_clustermap *tcmap, u8 * bitmap,
			  u64 blocks, int x1, int x2, int y1, int y2)
{
	u64 i;
	int line;

	if (!bitmap)
		fatal("no bitmap for term_clustermap()");

	tcmap->x1 = x1;
	tcmap->x2 = x2;
	tcmap->y1 = y1;
	tcmap->y2 = y2;

	tcmap->blocks = blocks;
	tcmap->bitmap = bitmap;

	tcmap->lenx = tcmap->x2 - tcmap->x1;
	tcmap->leny = tcmap->y2 - tcmap->y1;

	/* if terminal is too small, we exit! */
	if (tcmap->lenx < 10)
		fatal("lenx<10");
	if (tcmap->leny < 10)
		fatal("leny<10");

	tcmap->blocks_per_unit = tcmap->blocks / (tcmap->lenx * tcmap->leny);
	tcmap->blocks_per_unit += 1;
	tcmap->needed_units = tcmap->blocks / tcmap->blocks_per_unit;

	/* no work in progress */
	line = 0;
	for (i = 0; i < tcmap->needed_units; i++) {

		/* go to next line, if needed */
		if (i % tcmap->lenx == 0) {
			line++;
			term_gotoxy(tcmap->x1, tcmap->y1 + line);
		}

		term_blockmap_showunit(tcmap->bitmap, i,
				       tcmap->blocks_per_unit);
	}

	return;
}

/**
 * called every time the unit is changing... so bcurrent must be a multiple of tcmap->blocks_per_unit
 */
void term_clustermap(struct term_clustermap *tcmap, u64 bcurrent)
{
	static int mouth = 0;
	int unit, x, y;

	/* position before mounth ... */
	unit = bcurrent / tcmap->blocks_per_unit;
	for (x = unit - 1, y = 1; x >= tcmap->lenx; x -= tcmap->lenx)
		y++;
	if (unit) {
		term_gotoxy(tcmap->x1 + x, tcmap->y1 + y);
		term_s(" ");
	}

	/* position of mounth ... */
	for (x = unit, y = 1; x >= tcmap->lenx; x -= tcmap->lenx)
		y++;

	term_gotoxy(tcmap->x1 + x, tcmap->y1 + y);
	term_s(C_PACMAN);
	if (bcurrent == tcmap->blocks)
		term_s(" ");
	else if (mouth)
		term_s("C");
	else
		term_s("c");
	term_s(C_CLUSTER);

#if 0
	term_gotoxy(3, term_maxy - 4);
	term_print("unit=%d x=%d y=%d %U/%U....", unit, x, y, bcurrent,
		   tcmap->blocks);
	term_flush();
#endif

	/* flip/flap */
	mouth = !mouth;

	return;
}
