
/**
 * Copyright © 2010 - 2014 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* libowfat */
#include "io.h"
#include "case.h"

/* fribee-ng */
#include "die.h"
#include "splice.h"
#include "wrapper.h"
#include "transport.h"

#include "havesplice.h"

#ifdef HAVE_SPLICE

#define SPLICE_MAX	1024 * 64
#define SPLICE_FLAGS_2USE  (SPLICE_F_MOVE|SPLICE_F_NONBLOCK|SPLICE_F_MORE)

void tp_splice_clone(int fd_in, int dst_count, int64 * fds, s64 off, s64 bytes)
{
	int pfds[2], out;

	xpipe(pfds);

	/* XXX: each one should have an own thread! */
	for (out = 0; out < dst_count; out++) {
		int ofd = fds[out];
		s64 off_in = off;
		s64 off_out = off;
		s64 len = bytes;

		do {
			int todo = min(SPLICE_MAX, len);
			int in = splice_loop(fd_in, &off_in, pfds[1], 0,
					     todo, 0);

			if (in == 0)
				break;

			len -= todo;
			while (in > 0) {
				int out = splice_loop(pfds[0], 0, ofd,
						      &off_out, in, 0);

				if (out == 0)
					die_msg(111, "empty pipe?");
				in -= out;
			}
		}
		while (len);
	}

	close(pfds[0]);
	close(pfds[1]);

	return;
}

#endif
