
/**
 * Copyright © 2010 - 2014 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* libowfat */
#include "io.h"
#include "case.h"

/* fribee-ng */
#include "mem.h"
#include "die.h"
#include "splice.h"
#include "wrapper.h"
#include "transport.h"

#include "havesplice.h"

#ifdef HAVE_SPLICE

#define SPLICE_MAX	1024 * 64
#define SPLICE_FLAGS_2USE  (SPLICE_F_MOVE|SPLICE_F_NONBLOCK|SPLICE_F_MORE)
const char *errmsg = "splice-tee-loop error?!";
void tp_splice_tee_clone(int fd_in, int dst_count, int64 * fds, s64 off,
			 s64 bytes)
{
	s64 len = bytes;
	s64 off_in = off;
	s64 off_out = off;
	s64 out;		/* output counter */
	int pfds[2];		/* default pipe */
	int **tfds;		/* pipes for tee */

	xpipe(pfds);

	/* cout the output fd's and allocate space for them */
	tfds = mem_talloc(int *, dst_count);

	for (out = 0; out < dst_count; out++) {
		tfds[out] = mem_talloc(int, 2);

		xpipe(tfds[out]);
	}

	do {
		s64 todo = min(SPLICE_MAX, len);

		/* fill pfd[] pipe */
		s64 tlen = splice_loop(fd_in, &off_in, pfds[1], 0, todo,
				       SPLICE_FLAGS_2USE);

		if (unlikely(tlen == 0))
			break;
		if (tlen < todo)
			todo = tlen;

		/* clone content of pfd[] pipe via tee */
		if (out > 1)
			for (out = 1; out < dst_count; out++) {
				tlen = todo;
				off_out = off;
				tlen =
				    tee_loop(pfds[0], tfds[out][1], todo,
					     SPLICE_F_NONBLOCK);
				if (tlen != todo)
					die_msg(111, errmsg);
				tlen =
				    splice_loop(tfds[out][0], 0, fds[out],
						&off_out, todo,
						SPLICE_FLAGS_2USE);
				if (tlen != todo)
					die_msg(111, errmsg);
			}

		/* consume pfd[] pipe via splice */
		off_out = off;
		tlen =
		    splice_loop(pfds[0], 0, fds[0], &off_out, todo,
				SPLICE_FLAGS_2USE);
		if (tlen != todo)
			die_msg(111, errmsg);

		off = off_out;	/* new offset now */
		len -= todo;
	}
	while (len);

	close(pfds[0]);
	close(pfds[1]);

	/* free resources */
	for (out = 0; out < dst_count; out++) {
		close(tfds[out][0]);
		close(tfds[out][1]);
		mem_free(tfds[out]);
	}
	mem_free(tfds);

	return;
}

#endif
