
/**
 * Copyright © 2012 - 2014 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef TRANSPORT_H
#define TRANSPORT_H

/* system */
#include <sys/types.h>

/* libowfat */
#include "io.h"

/* frisbee */
#include "defs.h"
#include "fs.h"
#include "progress.h"

#include "havesplice.h"

/* valid transport modes for transport_doit() */
#define FNG_TMODE_READWRITE  1
#define FNG_TMODE_SPLICE     2
#define FNG_TMODE_SPLICE_TEE 3
#define FNG_TMODE_SENDFILE   4
#define FNG_TMODE_DUMMY      5
#define FNG_TMODE_DEFAULT    FNG_TMODE_SENDFILE

typedef void (tp_init) (int fd_in, int dst_count, int64 * fds);
typedef void (tp_clone) (int fd_in, int dst_count, int64 * fds, s64 start,
			 s64 bytes);
typedef void (tp_done) (int fd_in, int dst_count, int64 * fds);

/* dummy mode */
extern tp_init tp_dummy_init;
extern tp_clone tp_dummy_clone;
extern tp_done tp_dummy_done;

/* standard read write mode */
extern tp_clone tp_rdwr_clone;

/* sendfile mode */
extern tp_clone tp_sendfile_clone;

#ifdef HAVE_SPLICE
/* linux splice mode */
extern tp_clone tp_splice_clone;

/* linux splice-tee mode */
extern tp_clone tp_splice_tee_clone;
#endif

/* readtest: will not write, it just reads again the source and then reads
 * all destination fd's and verify them with the source */
extern tp_clone tp_readtest;

/* this is used for default values */
extern int find_transportmode(const char *name);

/**
 * transport_doit() - copy some data of a partition
 *
 * @fd:			source fd
 * @p:			information of source partition
 * @dst_count:		number of destinations
 * @fds:		destination fd's (opened in rw mode)
 * @transportmode:	type of transport (splice, tee, rdwr...)
 */
extern void transport_doit(int64 fd, struct partition *p, int dst_count,
			   int64 * fds, int transportmode, int redtest,
			   struct progress *pd, int part1, int partN,
			   int threads);

#endif
