
/**
 * Copyright © 2001 - 2014 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * debug section: 13
 */

#include "tty.h"

static struct termios told;
static int tty_init_done = 0;

int is_tty(int fd)
{
	struct winsize ws;

	if (ioctl(fd, TIOCGWINSZ, &ws) == -1) {
		return 0;
	}

	return 1;
}

void tty_init(void)
{
	struct termios tnew;

	if (tty_init_done) {
		return;
	}

	if (tcgetattr(0, &told) == -1) {
		die_term("tcgetattr()");
	}

	tty_init_done = 1;
	tnew = told;

	tnew.c_lflag &= ~(ICANON | ECHO | ECHOE);
	tnew.c_cc[VMIN] = 1;

	if (tcsetattr(0, TCSANOW, &tnew) == -1) {
		die_term("tcsetattr()");
	}

	return;
}

void tty_getsize(int *x, int *y)
{
	struct winsize ws;

	if (ioctl(1, TIOCGWINSZ, &ws) == -1) {
		*x = *y = 0;
		return;
	}

	*x = ws.ws_col;
	*y = ws.ws_row;

	return;
}

void tty_restore(void)
{
	if (tty_init_done) {
		if (tcsetattr(0, TCSANOW, &told) == -1) {
			die_term("tcsetattr()");
		}
	}

	tty_init_done = 0;

	return;
}
