
/**
 * Copyright © 2010 - 2016 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FNG_WRAPPER_H
#define FNG_WRAPPER_H

/* system */
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>

/* libowfat */
#include "io.h"
#include "stralloc.h"

/* frisbee-ng */
#include "defs.h"
#include "die.h"
#include "haveinline.h"

/* on error, -1 will be returned */
extern s64 read_loop(int fd, void *buf, u64 count);
extern s64 write_loop(int fd, const void *buf, u64 count);

/* on error, these two functions call fatal() */
extern void read_loop2(int fd, void *buf, u64 count);
extern void write_loop2(int fd, const void *buf, u64 count);

/* on error, -1 will be returned */
extern s64 pread_loop(int fd, void *buf, u64 count, u64 off);
extern s64 pwrite_loop(int fd, const void *buf, u64 count, u64 off);

/* on error, these two functions call die_msg() */
extern s64 pread_loop2(int fd, void *buf, u64 count, u64 off);
extern s64 pwrite_loop2(int fd, const void *buf, u64 count, u64 off);

/**
 * on problems, these two will exit
 */
extern s64 pread_sa(int fd, stralloc * sa, u64 count, u64 off);
extern s64 pwrite_sa(int fd, stralloc * sa, u64 off);
extern s64 pwrite_sa_append(int fd, stralloc * sa, u64 off);

/* on error, xpipe() calls die_msg() */
extern int xpipe(int pfd[2]);

static inline void io_fd2(int64 fd)
{
	if (!io_fd(fd))
		fatal("io_fd() failed");
}

#endif
